// Fuses fuer Arduino Nano V3: LV: 0xFF, HV: 0xD4, EV: 0xFD, LB: 0xFF
// Optiboot Bootloader flashen!!
// Achtung: Der hat 115200 Bd, default bei Arduino ist 57600 Bd -> boards.txt aendern (Z. 147)
// Vorteil des Optiboot: Sehr kurze Neuanlaufzeit nach Watchdog-Reset (program hang)
//
//-----------------//
// QuickmillSaver3 //
//-----------------//
// Der Quickmill - 5000A Kaffee-Vollautomat hat leider keine Ueberwachung fuer 
// zu wenig Wasser oder zu vollen Kaffeesatzbehaelter.
// In Folge passiert es haeufiger, dass entweder die Tasse nicht voll wird, weil
// die Durchflussueberwachung wegen zu wenig Wasser den Bruehvorgang stoppt. Dass
// dabei die Pumpe haessliche Geraeusche von sich gibt, weil sie auf Block geht, 
// kommt nervend dazu.
// Oder der Kaffeesatz staut sich im Innern des Technikraums bis an die Oberkante,
// weil vergessen wurde, den Satzbehaelter rechtzeitig zu leeren.
//
// Folglich muss diese Ueberwachung zusaetzlich eingebaut werden.
// Ein OLED-Display mit 0,96 Zoll Diagonale und I2C-Anschluss wird von einem 
// Arduino Nano V3 angesteuert und visualisiert sowohl den Wasserstand als auch 
// den Fuellgrad des Satzbehaelters.
// Optional wird wahrend des Bruehvorgangs eine stilisierte Tasse gezeigt, die 
// sich langsam fuellt. Die Zeit von leer bis voll ist parametrierbar und laeuft
// frei parallel zum Fuellvorgang (nicht von der Hardware gefuehrt).
// Die Ansteuerung des OLED-Display erfolgt ueber die von Adafruit zur Verfuegung
// gestellten Library (Text und Grafik).
// 
// Da die Gefahr besteht, dass Ultraschallsensoren (QuickmillSaver V1) in 
// der feuchten Umgebung oberhalb des Wasser- und des Satzbehaelters vergammeln, 
// wird ein Tassenzaehlers implementiert, der jede gezapfte Tasse zaehlt.
// Nach einer bestimmten Anzahl Tassen wird davon ausgegangen, dass der Wasserbehaelter 
// nachzufuellen ist. 
// Nach einer bestimmten anderen Anzahl Tassen wird davon ausgegangen, dass der 
// Satzbehaelter voll und somit zu leeren ist. 
// Diese Zaehler muessen im EEPROM gefuehrt werden, da der Kaffeeautomat im 
// ausgeschalteten Zustand komplett stromlos ist.
//
// Das EEPROM im AtMega hat 100000 zugesicherte Schreibzyklen, das duerfte die zu 
// erwartende Restlaufzeit des Kaffeeautomaten weit ueberschreiten, eine Strategie
// zur Verteilung der Schreibzyklen auf die Speicherzellen eruebrigt sich also.
// Anhaltspunkt: In den ersten 15 Jahren wurden rund 5000 Tassen gezapft. 
// Der Tassenzaehler ist unsigned int (65535) und reicht somit noch 100 Jahre...
// Falls doch die verwendeten EEPROM-Zellen vorher den Geist aufgeben, kann das 
// #define EESTARTADRESSE geaendert werden. Dabei werden sinnvolle Defaults gesetzt.
// Danach sind die korrekten Einstellungen neu zu setzen (-> Menue).
//
// Um zu erkennen wenn der Wasser- oder der Satzbehaelter "bedient" wird, wird
// unter jedem Behaelter ein kleiner Magnet eingeklebt, der jeweils einen Hallsensor
// vom Typ "A3144" betaetigt, der den zugehoerigen Zaehler wieder ruecksetzt.
// Die Hallsensoren finden Platz unterhalb der Bodenplatte.
// Der Sensor fuer die Erkennung eines Zapf-Vorgangs nutzt den bereits vorhandenen 
// Magneten auf der Achse des Getriebes und wird über einen Adapter aus Kunststoff
// an den beiden vorhandenen Sensoren fuer obere und untere Endlage befestigt.
//
// Die verschiedenen Zaehler und Grenzwerte sind einstellbar, die Bedienung 
// erfolgt mit Touchsensoren (z.B. https://www.ebay.de/itm/253381740902) durch die 
// Tuer des Technikraumes hindurch.
// Die Sensoren arbeiten durch 3 mm Kunststoff hindurch, so dass keine
// Loecher fuer die Bedienung im Gehaeuse notwendig sind.
// Alternativ arbeitet das Programm auch mit einem Kreuztasten-Joystick. 
// Die Sensortasten koennen auch ueber einen QT1070 von Atmel ueber I2C angeschlossen werden.
// Das ist momentan implementiert, wird über #define USEQTOUCH aktiviert.
// Dafuer wird die Library "Seeed_QTouch" von Seeed Studio, https://github.com/Seeed-Studio/Seeed_QTouch
// verwendet.
// Achtung!
// Der Returnwert von QTouch.isTouch() ist nicht 0 und 1, wie im Beispiel beschrieben!
// Mit "if(QTouch.isTouch(QTRIGHT) == 1)" wird der Tastendruck nicht erkannt, mit 
// "if(QTouch.isTouch(QTRIGHT))" hingegen schon. Die Abfrage auf _nicht_ gedrueckte Taste ist somit
// "if(!QTouch.isTouch(QTRIGHT))".
//
// Es werden vier Touchsensoren eingebaut, rund um das Display herum, an jeder Seite
// einer (Hoch/Runter/Links/Rechts).
// Bedienung
// Nicht im Menue - Rechts: Menue wird gestartet
// Im Menue - kurz Rechts: Parameter werden reihum zum Aendern aktiviert
// Im Menue - Hoch/Runter: Parameter werden geaendert (Wasser (12..18), Satz (25..35), Tassen (5000..65535),
//						   Tassengrafik anzeigen (Ja/Nein), T bis Voll (10..59), T nach Voll (1..9)
// Im Menue - lange Rechts >1s: Parameter Tassen (gesamt) wird aktiviert, hoch/runter aendert kontinuierlich
//						   Das wird normalerweise nur einmal nach dem Einbau des Display benötigt um 
//						   die Zaehler aufeinander abzustimmen (der Tassenzaehler im Arduino ist Kuer, 
//						   die Ueberwachung der Behaelter funktioniert auch ohne diesen Gesamtzaehler)
//
// Der Atmel hat noch Platz fuer eine Animation einer sich fuellenden Tasse sowie einen sporadisch
// anstelle der Balkengrafik auftauchenden Smiley ;-) (momentan 10 verschiedene Smileys, zufaellig selektiert)
// Der Randomgenerator (seed) wird mit der Tassenzahl randomisiert.
// Der Smiley kommt immer, die animierte Tasse kann im Menu Ein/Aus geschaltet werden. Die verschiedenen
// Zeiten vor Bezug-Start, die Dauer des Bezugs sowie die Pause mit Dampf danach sind parametrierbar, so dass der 
// optische Effekt auf die Hardware abgeglichen werden kann (echte Tasse voll -> Grafik voll, gleichzeitig)
//
// 21.06.2018, V1.0
// - Reihenfolge von Tassenanimation und Warnung ueber Fuellstaende vertauscht. 
//   Jetzt wird zuerst die Tasse animiert - die Tasse wird ja in Realitaet ebenfalls zuerst gefuellt - dann 
//   wird die Behaelterwarnung angezeigt.
// - Programmlogik dafuer ziemlich umgekrempelt und in Subroutines verpackt.
//
// 26.11.2020, V1.1
// - Nach 2 Minuten wird das Display gedimmt um das Einbrennen der Rahmen für die Füllstaende zu minimieren.
//   Bei der naechsten Zapfung oder beim Aufruf des Menu wird die Helligkeit auf wieder Normal gestellt.
// - Die Warnungen fuer Wasserstand oder Satzfuellstand werden langsam pulsierend angezeigt.
// - Umstieg auf neue Adafruit OLED-Lib (neuer Constructor mit "width, height").
//
// 27.11.2020, V1.2
// - Version und Build Datum in den Einstellungen anzeigen.
// - Display Pulsiergeschwindigkeit bei Warnung erhoeht.
// - Darstellung der QTouchWerte in Einstellungen unterdrueckt, wenn Tasten oder Einzelsensoren verwendet werden.
//

// ----------------------------------------------------------------------------------------------------------------------------
// Version
// ----------------------------------------------------------------------------------------------------------------------------
#define VERSION		"1.2"
#define BUILDDATE	"27.11.2020"
// ----------------------------------------------------------------------------------------------------------------------------
// Tasten/Touchsensoren
// ----------------------------------------------------------------------------------------------------------------------------
// Kommentar bei "#define QTOUCHKEYS" entfernen, wenn die Bedienung ueber QTouch-Controller erfolgt
#define QTOUCHKEYS
// ----------------------------------------------------------------------------------------------------------------------------
#ifdef QTOUCHKEYS
	// nachfolgende #defines gelten, wenn die Bedienung ueber den I2C QTouch-Controller erfolgt
	#pragma message("QTouch Sensoren werden verwendet...")	// Informative Meldung beim Compilieren
	#define QTUP 2				// Anschluss der Tasten am QTouch-Controller 
	#define QTDOWN 0			// entspricht auch dem TastenCode xx, gelesen mit "QTouch.isTouch(xx)"
	#define QTLEFT 1
	#define QTRIGHT 3
	#define GETKEYMACRO(RICHTUNG) (QTouch.isTouch(RICHTUNG))		// je nach Typ der Bedienung
#else
	// nachfolgende #defines gelten, wenn die Bedienung ueber Tasten (Joystick oder auch einzelne 
	// Touch-Sensoren (siehe Link oben)) erfolgt
	#pragma message("Normale Tasten werden verwendet...")	// Informative Meldung beim Compilieren
	#define QTUP 8				// DataPin der Tasten am Arduino 
	#define QTDOWN 11
	#define QTLEFT 12
	#define QTRIGHT 10
	#define GETKEYMACRO(RICHTUNG) (digitalRead(RICHTUNG) == LOW)	// je nach Typ der Bedienung
#endif	
// ----------------------------------------------------------------------------------------------------------------------------
// Testmode	(Tassenzahlen im Menue verringert um beim Entwickeln schnellere Turnarounds zu erreichen
// #define TESTMODE 1


// benoetigte Bibliotheken
#include <SPI.h>					// das Display ist ueber I2C angeschlossen
#include <Wire.h>					// das Display ist ueber I2C angeschlossen
#include <gfxfont.h>				// der Standard-Font fuer OLED-Displays
#include <Adafruit_GFX.h>			// Grafik-Lib von Adafruit fuer OLED-Displays
#include <Adafruit_SSD1306.h>		// low level Treiber von Adafruit fuer OLED-Displays
#include <EEPROM.h>					// die Werte werden im EEPROM gespeichert
#include <avr/eeprom.h>				// die Werte werden im EEPROM gespeichert
#include <Seeed_QTouch.h>			// Lib von Seeed Studio fuer QT1070 Touch-Sensor
#include <avr/wdt.h>				// Watchdog Timer als Notnagel

// OLED instantiieren
Adafruit_SSD1306 display(128, 64);	// new constructor as of Version 1.2 (November 2018), 2020-11-26

#define TEXTTOP 18					// linke obere Ecke der Textausgaben im blauen Teil (das Display ist zweifarbig)
#define TEXTLEFT 0

#define DIMTIMEOUT 120000			// 2 Minuten bis das Display gedimmt wird (Abnutzung/Einbrennen minimieren), 2020-11-26

#define ZEILE1 TEXTTOP-16			// das Display kann mit Font 1 (klein) 5 Zeilen darstellen
#define ZEILE2 TEXTTOP-2
#define ZEILE3 TEXTTOP+10
#define ZEILE4 TEXTTOP+22
#define ZEILE5 TEXTTOP+34

#define TOPLEFTWASSER 12			// linke obere Ecke des Rahmens fuer Wasser (X-Koordinate)
#define TOPLEFTSATZ 76				// linke obere Ecke des Rahmens fuer Satz (X-Koordinate)
#define TOPLEFTSMILEY 40			// linke obere Ecke der Smileys (X-Koordinate)

#define LED13 13
#define EESTARTADRESSE 1000			// Startadresse fuer die Speicherung der Tassenzahlen im EEPROM
									// Debug und Entwicklung: 1000, kann geaendert werden, z.B. 0 (Null)

// die beiden nachfolgenden Werte koennen per Menue geaendert werden -> EEPROM
// defines nur fuer Debug/Entwicklung
#define TASSENADDWASSER  14			// maximale Anzahl Tassen bevor Wasser nachzufuellen ist
#define TASSENREMOVESATZ 31			// maximale Anzahl Tassen bevor Kaffeesatz geleert werden muss

#define GELBELED A0					// Ausgang gelbe LED, Debug-Hilfe, zur Laufzeit nicht verwendet

#define TASSENSENSOR 9				// Eingang des Hallsensors am Tassenzaehler
#define WASSERSENSOR 5				// Eingang des Hallsensors am Wassersensor
#define SATZSENSOR 3				// Eingang des Hallsensors am Satzsensor

// Tasten Defines
#define KEYUP 8						// Arduino Pins der Tasten (wird fuer pinMode() benoetigt)
#define KEYDOWN 11
#define KEYLEFT 12
#define KEYRIGHT 10

#define TESTOUT A2					// Ausgang zum Messen von Schleifenzeiten usw.

//{	// Bitmap fuer Kaffeetasse
// 'Kaffeetasse_leer', 64x36px  (64x40px)
const unsigned char KaffeeTasse [] PROGMEM = {
	// 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	// 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 
	0x03, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x81, 0xf0, 
	0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x9f, 0xfc, 0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0xf8, 0x0e, 
	0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0xe0, 0x0e, 0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0xe0, 0x0e, 
	0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x80, 0x0e, 0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x80, 0x0e, 
	0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x80, 0x0e, 0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x80, 0x0e, 
	0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x80, 0x0e, 0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x80, 0x1c, 
	0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x80, 0x38, 0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x80, 0x38, 
	0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x80, 0xf0, 0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x83, 0xe0, 
	0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0x9f, 0x80, 0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0xfe, 0x00, 
	0x03, 0x80, 0x00, 0x00, 0x00, 0x03, 0xf0, 0x00, 0x03, 0xc0, 0x00, 0x00, 0x00, 0x07, 0xc0, 0x00, 
	0x03, 0xc0, 0x00, 0x00, 0x00, 0x07, 0x80, 0x00, 0x01, 0xe0, 0x00, 0x00, 0x00, 0x0f, 0x00, 0x00, 
	0x01, 0xe0, 0x00, 0x00, 0x00, 0x0f, 0x00, 0x00, 0x00, 0xf0, 0x00, 0x00, 0x00, 0x1e, 0x00, 0x00, 
	0x00, 0xf0, 0x00, 0x00, 0x00, 0x1e, 0x00, 0x00, 0x00, 0x78, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 
	0x00, 0x78, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0x00, 0x78, 0x00, 0x00, 
	0x3f, 0x1f, 0xff, 0xff, 0xff, 0xf1, 0xf8, 0x00, 0x0f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe0, 0x00, 
	0x03, 0xff, 0xff, 0xff, 0xff, 0xff, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
};
// 'Dampf1', 16x16px
const unsigned char Dampf1 [] PROGMEM = {
	0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x02, 0x00, 0x03, 0x20, 0x07, 0x30, 0x06, 0x30, 0x0e, 0x70, 
	0x0c, 0x70, 0x0c, 0xf0, 0x04, 0xe0, 0x04, 0xe0, 0x00, 0xc0, 0x00, 0x40, 0x00, 0x40, 0x00, 0x00
};
// 'Dampf2', 16x16px
const unsigned char Dampf2 [] PROGMEM = {
	0x00, 0x80, 0x03, 0x00, 0x06, 0x00, 0x0e, 0x00, 0x0e, 0x00, 0x0e, 0x00, 0x0e, 0x00, 0x0e, 0x10, 
	0x0f, 0x20, 0x07, 0x30, 0x07, 0xb0, 0x03, 0x90, 0x03, 0x90, 0x03, 0x90, 0x03, 0x20, 0x00, 0x00
};

// Array mit Linien zum Fuellen der Tasse
// writeFastHLine(int16_t x, int16_t y, int16_t w, uint16_t color);
const int16_t FuellLinien [] PROGMEM = {
//   X    Y  len
	40 , 59,  34,	// Satz 0
	40 , 58,  34,	// Satz 1
	39 , 57,  35,	// Satz 2
	39 , 56,  35,	// Satz 3
	38 , 55,  36,	// Satz 4
	
	38 , 54,  40,	// Satz 5
	38 , 53,  40,	// Satz 6
	38 , 52,  40,	// Satz 7
	38 , 51,  40,	// Satz 8
	38 , 50,  40,	// Satz 9
	38 , 49,  40,	// Satz 10
	38 , 48,  40,	// Satz 11
	38 , 47,  40,	// Satz 12
	38 , 46,  40,	// Satz 13
	38 , 45,  40,	// Satz 14
	38 , 44,  40,	// Satz 15
	38 , 43,  40,	// Satz 16
	38 , 42,  40,	// Satz 17
	38 , 41,  40,	// Satz 18
	38 , 40,  40,	// Satz 19
	38 , 39,  40,	// Satz 20
	38 , 38,  40,	// Satz 20
	38 , 37,  40,	// Satz 21
	38 , 36,  40,	// Satz 22
	38 , 35,  40,	// Satz 23
	38 , 34,  40,	// Satz 24
	38 , 33,  40,	// Satz 25
	38 , 32,  40	// Satz 26
};
unsigned int iLineCounter = 0;		// welche Linie wird angezeigt
#define TASSEZEILEN 30				// Anzahl der zu fuellenden Zeilen in der Tasse
//}

//{	// Bitmaps fuer Smileys
// 'Smiley1_bw', 48x48px
const unsigned char Smiley1_bw [] PROGMEM = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0xf8, 
	0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x03, 0xe0, 0x07, 0xc0, 0x00, 0x00, 0x0f, 
	0x00, 0x00, 0xf0, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x38, 0x00, 0x00, 0x70, 0x00, 0x00, 0x0e, 0x00, 
	0x00, 0xe0, 0x00, 0x00, 0x07, 0x00, 0x01, 0xc0, 0x60, 0x07, 0x03, 0x80, 0x01, 0x80, 0x70, 0x07, 
	0x01, 0x80, 0x03, 0x00, 0xf0, 0x0f, 0x00, 0xc0, 0x07, 0x00, 0xf0, 0x0f, 0x80, 0x60, 0x06, 0x00, 
	0xf0, 0x0f, 0x80, 0x60, 0x0c, 0x00, 0xf0, 0x0f, 0x80, 0x30, 0x0c, 0x00, 0xf8, 0x0f, 0x80, 0x30, 
	0x18, 0x00, 0xf0, 0x0f, 0x80, 0x18, 0x18, 0x00, 0xf0, 0x0f, 0x80, 0x18, 0x18, 0x00, 0xf0, 0x0f, 
	0x00, 0x18, 0x10, 0x00, 0x70, 0x07, 0x00, 0x0c, 0x30, 0x00, 0x60, 0x07, 0x00, 0x0c, 0x30, 0x00, 
	0x20, 0x02, 0x00, 0x0c, 0x30, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x30, 0x00, 0x00, 0x00, 0x00, 0x0c, 
	0x31, 0xc0, 0x00, 0x00, 0x03, 0x8c, 0x33, 0x80, 0x00, 0x00, 0x01, 0x0c, 0x30, 0x80, 0x00, 0x00, 
	0x01, 0x0c, 0x30, 0x80, 0x00, 0x00, 0x01, 0x0c, 0x18, 0xc0, 0x00, 0x00, 0x01, 0x18, 0x18, 0x40, 
	0x00, 0x00, 0x02, 0x18, 0x18, 0x40, 0x00, 0x00, 0x02, 0x18, 0x1c, 0x20, 0x00, 0x00, 0x04, 0x18, 
	0x0c, 0x30, 0x00, 0x00, 0x04, 0x30, 0x0c, 0x10, 0x00, 0x00, 0x08, 0x30, 0x06, 0x08, 0x00, 0x00, 
	0x10, 0x60, 0x03, 0x04, 0x00, 0x00, 0x20, 0xe0, 0x03, 0x83, 0x00, 0x00, 0xc0, 0xc0, 0x01, 0x80, 
	0xc0, 0x03, 0x81, 0x80, 0x00, 0xc0, 0x78, 0x1e, 0x03, 0x00, 0x00, 0x70, 0x07, 0xe0, 0x0e, 0x00, 
	0x00, 0x38, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x1e, 0x00, 0x00, 0x78, 0x00, 0x00, 0x07, 0x80, 0x01, 
	0xe0, 0x00, 0x00, 0x03, 0xf0, 0x0f, 0xc0, 0x00, 0x00, 0x00, 0x7f, 0xfe, 0x00, 0x00, 0x00, 0x00, 
	0x07, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
// 'Smiley2_bw', 48x48px
const unsigned char Smiley2_bw [] PROGMEM = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3f, 0xfc, 0x00, 0x00, 0x00, 0x01, 0xf8, 0x0f, 
	0x80, 0x00, 0x00, 0x07, 0x80, 0x01, 0xe0, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x38, 0x00, 0x00, 0x30, 
	0x00, 0x00, 0x0c, 0x00, 0x00, 0x60, 0x00, 0x00, 0x06, 0x00, 0x00, 0xc0, 0x00, 0x00, 0x03, 0x00, 
	0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 0x03, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x06, 0x00, 0x00, 0x00, 
	0x00, 0x60, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x30, 0x08, 0x00, 0xe0, 0x00, 0x00, 0x10, 0x18, 0x01, 
	0xe0, 0x00, 0x00, 0x18, 0x10, 0x01, 0xf0, 0x1f, 0x80, 0x08, 0x30, 0x01, 0xf0, 0x3f, 0xe0, 0x0c, 
	0x30, 0x01, 0xf0, 0x60, 0x30, 0x0c, 0x20, 0x01, 0xf0, 0x00, 0x10, 0x04, 0x60, 0x01, 0xe0, 0x00, 
	0x00, 0x06, 0x60, 0x00, 0xe0, 0x00, 0x00, 0x06, 0x60, 0x00, 0x00, 0x00, 0x00, 0x06, 0x40, 0x00, 
	0x00, 0x00, 0x00, 0x02, 0x40, 0x00, 0x00, 0x00, 0x00, 0x02, 0x40, 0x00, 0x00, 0x00, 0x00, 0x02, 
	0x40, 0x00, 0x00, 0x00, 0x00, 0x02, 0x40, 0x00, 0x00, 0x00, 0x00, 0x02, 0x40, 0x08, 0x00, 0x00, 
	0x10, 0x02, 0x40, 0x08, 0x00, 0x00, 0x18, 0x02, 0x60, 0x18, 0x00, 0x00, 0x18, 0x06, 0x61, 0xf0, 
	0x00, 0x00, 0x0f, 0x86, 0x20, 0xf0, 0x00, 0x00, 0x0f, 0x04, 0x30, 0x18, 0x00, 0x00, 0x18, 0x0c, 
	0x30, 0x18, 0x00, 0x00, 0x18, 0x0c, 0x10, 0x0c, 0x00, 0x00, 0x30, 0x08, 0x18, 0x06, 0x00, 0x00, 
	0x60, 0x18, 0x08, 0x03, 0x80, 0x01, 0xc0, 0x10, 0x0c, 0x01, 0xe0, 0x07, 0x80, 0x30, 0x06, 0x00, 
	0xff, 0xff, 0x00, 0x60, 0x03, 0x00, 0x3f, 0xfc, 0x00, 0xc0, 0x01, 0x80, 0x03, 0xc0, 0x01, 0x80, 
	0x00, 0xc0, 0x00, 0x00, 0x03, 0x00, 0x00, 0x60, 0x00, 0x00, 0x06, 0x00, 0x00, 0x30, 0x00, 0x00, 
	0x0c, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x38, 0x00, 0x00, 0x07, 0x80, 0x01, 0xe0, 0x00, 0x00, 0x01, 
	0xf8, 0x0f, 0x80, 0x00, 0x00, 0x00, 0x3f, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
// 'Smiley5', 48x48px
const unsigned char Smiley5_bw [] PROGMEM = {
	0x00, 0x00, 0x0f, 0xf0, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x03, 0xc0, 0x03, 
	0xc0, 0x00, 0x00, 0x0e, 0x00, 0x00, 0x70, 0x00, 0x00, 0x38, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x60, 
	0x00, 0x00, 0x0e, 0x00, 0x00, 0xc0, 0x00, 0x00, 0x03, 0x00, 0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 
	0x03, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x06, 0x00, 0x00, 0x00, 0x00, 0x60, 0x0c, 0x00, 0x00, 0x00, 
	0x00, 0x20, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x30, 0x18, 0x00, 0x80, 0x00, 0x00, 0x18, 0x10, 0x01, 
	0xc0, 0x00, 0x00, 0x08, 0x30, 0x03, 0xe0, 0x00, 0x78, 0x0c, 0x20, 0x03, 0xe0, 0x01, 0xc0, 0x0c, 
	0x20, 0x03, 0xf0, 0x03, 0x00, 0x04, 0x60, 0x07, 0xf0, 0x07, 0xf0, 0x06, 0x60, 0x07, 0xf0, 0x0f, 
	0xf8, 0x06, 0x40, 0x03, 0xf0, 0x08, 0x00, 0x02, 0x40, 0x03, 0xe0, 0x00, 0x00, 0x02, 0x40, 0x03, 
	0xe0, 0x00, 0x00, 0x02, 0x40, 0x01, 0xe0, 0x00, 0x00, 0x02, 0xc0, 0x00, 0x80, 0x00, 0x00, 0x02, 
	0xc0, 0x00, 0x00, 0x00, 0x10, 0x02, 0x40, 0x00, 0x00, 0x00, 0x10, 0x02, 0x40, 0x00, 0x00, 0x00, 
	0x18, 0x02, 0x40, 0x00, 0x00, 0x00, 0x0c, 0x02, 0x40, 0x00, 0x00, 0x00, 0x1f, 0x06, 0x60, 0x00, 
	0x00, 0x00, 0x30, 0x06, 0x20, 0x00, 0x00, 0x00, 0x20, 0x04, 0x20, 0x10, 0x00, 0x00, 0x60, 0x04, 
	0x30, 0x10, 0x00, 0x00, 0xc0, 0x0c, 0x10, 0x08, 0x00, 0x01, 0x80, 0x08, 0x18, 0x04, 0x00, 0x03, 
	0x00, 0x18, 0x08, 0x06, 0x00, 0x0e, 0x00, 0x30, 0x0c, 0x01, 0xc0, 0xf8, 0x00, 0x30, 0x06, 0x00, 
	0x7f, 0xc0, 0x00, 0x60, 0x03, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 
	0x00, 0xc0, 0x00, 0x00, 0x03, 0x00, 0x00, 0x60, 0x00, 0x00, 0x06, 0x00, 0x00, 0x38, 0x00, 0x00, 
	0x1c, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x70, 0x00, 0x00, 0x07, 0x80, 0x01, 0xe0, 0x00, 0x00, 0x01, 
	0xf8, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x1f, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
// 'Smiley6', 48x48px
const unsigned char Smiley6_bw [] PROGMEM = {
	0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x0f, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x7f, 0xff, 0xff, 
	0xfe, 0x00, 0x01, 0xff, 0xc0, 0x03, 0xff, 0x80, 0x03, 0xf8, 0x0f, 0xf0, 0x1f, 0xc0, 0x03, 0xc0, 
	0x7f, 0xfe, 0x03, 0xc0, 0x07, 0x83, 0xff, 0xff, 0xc1, 0xe0, 0x07, 0x8f, 0xfc, 0x3f, 0xf1, 0xe0, 
	0x03, 0xff, 0xc0, 0x03, 0xff, 0xc0, 0x01, 0xfe, 0x00, 0x00, 0x7f, 0x80, 0x00, 0xff, 0xf0, 0x0f, 
	0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x01, 0xe7, 0xff, 0xff, 0xe7, 0x80, 0x03, 0xc0, 
	0x1f, 0xf8, 0x03, 0xc0, 0x07, 0x80, 0x00, 0x00, 0x01, 0xe0, 0x07, 0x80, 0x00, 0x00, 0x01, 0xe0, 
	0x0f, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x70, 0x1e, 0x00, 0x00, 0x00, 
	0x00, 0x78, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x78, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x38, 0x1c, 0x00, 
	0xe0, 0x07, 0x00, 0x38, 0x3c, 0x03, 0xf8, 0x1f, 0xc0, 0x3c, 0x38, 0x03, 0xfc, 0x3f, 0xc0, 0x3c, 
	0x38, 0x07, 0xbc, 0x3d, 0xe0, 0x1c, 0x38, 0x07, 0x1c, 0x38, 0xe0, 0x1c, 0x38, 0x00, 0x00, 0x00, 
	0x00, 0x1c, 0x38, 0x00, 0x00, 0x00, 0x00, 0x1c, 0x3c, 0x00, 0x00, 0x00, 0x00, 0x3c, 0x3c, 0x00, 
	0x00, 0x00, 0x00, 0x3c, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x38, 0x1c, 0x0c, 0x00, 0x00, 0x30, 0x38, 
	0x1c, 0x1e, 0x00, 0x00, 0x78, 0x38, 0x1e, 0x0f, 0x00, 0x00, 0xf0, 0x78, 0x0e, 0x07, 0xc0, 0x03, 
	0xe0, 0x70, 0x0f, 0x03, 0xf0, 0x0f, 0xc0, 0xf0, 0x07, 0x81, 0xff, 0xff, 0x81, 0xe0, 0x07, 0x80, 
	0x7f, 0xfe, 0x01, 0xe0, 0x03, 0xc0, 0x1f, 0xf8, 0x03, 0xc0, 0x01, 0xe0, 0x00, 0x00, 0x07, 0x80, 
	0x00, 0xf0, 0x00, 0x00, 0x0f, 0x00, 0x00, 0x7c, 0x00, 0x00, 0x3e, 0x00, 0x00, 0x3e, 0x00, 0x00, 
	0x7c, 0x00, 0x00, 0x1f, 0x80, 0x01, 0xf8, 0x00, 0x00, 0x0f, 0xf8, 0x1f, 0xf0, 0x00, 0x00, 0x03, 
	0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x0f, 0xf0, 0x00, 0x00
};
// 'Smiley7', 48x48px
const unsigned char Smiley7_bw [] PROGMEM = {
	0x00, 0x00, 0x1f, 0xf8, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x07, 0xff, 0xff, 
	0xe0, 0x00, 0x00, 0x1f, 0xfe, 0x7f, 0xf0, 0x00, 0x00, 0x3f, 0xc0, 0x03, 0xfc, 0x00, 0x00, 0x7e, 
	0x00, 0x00, 0x7e, 0x00, 0x00, 0xf8, 0x00, 0x00, 0x1f, 0x00, 0x01, 0xf0, 0x00, 0x00, 0x0f, 0x80, 
	0x03, 0xe0, 0x00, 0x00, 0x07, 0xc0, 0x07, 0x80, 0x00, 0x00, 0x03, 0xe0, 0x0f, 0x80, 0x00, 0x00, 
	0x01, 0xf0, 0x1f, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x78, 0x3c, 0x00, 
	0x00, 0x00, 0x00, 0x38, 0x3f, 0xff, 0xf0, 0x0f, 0xff, 0xfc, 0x7f, 0xff, 0xfb, 0xdf, 0xff, 0xfc, 
	0x7f, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x77, 0x00, 0x1f, 0xf8, 
	0x00, 0xee, 0xf7, 0x00, 0x1e, 0x78, 0x00, 0xef, 0xf7, 0x00, 0x1c, 0x38, 0x00, 0xef, 0xf7, 0x00, 
	0x1c, 0x38, 0x00, 0xef, 0xf7, 0x00, 0x1c, 0x38, 0x00, 0xef, 0xe7, 0x00, 0x3c, 0x3c, 0x01, 0xef, 
	0xe7, 0x80, 0x3c, 0x3c, 0x01, 0xef, 0xf3, 0xc0, 0x78, 0x1e, 0x03, 0xcf, 0xf3, 0xe1, 0xf8, 0x1f, 
	0x07, 0xcf, 0xf1, 0xff, 0xf0, 0x0f, 0xff, 0x8f, 0xf0, 0xff, 0xe0, 0x07, 0xff, 0x0f, 0x70, 0x3f, 
	0x80, 0x01, 0xfc, 0x0e, 0x70, 0x00, 0x00, 0x00, 0x00, 0x1e, 0x78, 0x00, 0x00, 0x00, 0x00, 0x1e, 
	0x38, 0x03, 0x80, 0x01, 0xc0, 0x1c, 0x3c, 0x07, 0xc0, 0x03, 0xe0, 0x3c, 0x3c, 0x03, 0xe0, 0x07, 
	0xc0, 0x3c, 0x1e, 0x01, 0xf0, 0x0f, 0x80, 0x78, 0x1f, 0x00, 0xff, 0xff, 0x00, 0xf0, 0x0f, 0x80, 
	0x7f, 0xfe, 0x01, 0xf0, 0x07, 0x80, 0x3f, 0xf8, 0x03, 0xe0, 0x03, 0xe0, 0x07, 0xe0, 0x07, 0xc0, 
	0x01, 0xf0, 0x00, 0x00, 0x0f, 0x80, 0x00, 0xf8, 0x00, 0x00, 0x1f, 0x00, 0x00, 0x7e, 0x00, 0x00, 
	0x7e, 0x00, 0x00, 0x3f, 0xc0, 0x03, 0xfc, 0x00, 0x00, 0x1f, 0xfe, 0x7f, 0xf0, 0x00, 0x00, 0x07, 
	0xff, 0xff, 0xe0, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x1f, 0xf8, 0x00, 0x00
};
// 'Smiley8', 48x48px
const unsigned char Smiley8_bw [] PROGMEM = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x60, 0x0c, 0x00, 0x00, 
	0x00, 0x00, 0xc0, 0x0c, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x1c, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x1c, 
	0x00, 0x00, 0x00, 0x00, 0xf8, 0x1f, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x0f, 0x00, 0x00, 0x00, 0x00, 
	0x70, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x60, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x20, 0x00, 0x00, 0x04, 0x00, 0x00, 0x60, 0x00, 0x00, 0x06, 0x00, 0x00, 0xe0, 0x00, 0x00, 
	0x07, 0x00, 0x00, 0xb0, 0x00, 0x00, 0x0d, 0x00, 0x00, 0x18, 0x00, 0x00, 0x18, 0x00, 0x00, 0x0c, 
	0x00, 0x00, 0x30, 0x00, 0x00, 0x06, 0x00, 0x00, 0x70, 0x00, 0x00, 0x01, 0x80, 0x01, 0xc8, 0x00, 
	0x00, 0x00, 0xf8, 0x1f, 0x08, 0x00, 0x00, 0x00, 0x1f, 0xfa, 0x08, 0x00, 0x00, 0x00, 0x00, 0x02, 
	0x08, 0x00, 0x00, 0x00, 0x00, 0x01, 0xb0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
// 'Smiley9', 48x48px
const unsigned char Smiley9_bw [] PROGMEM = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0x00, 0x00, 0x30, 0x1f, 0x00, 0x00, 0x00, 0x00, 
	0xf0, 0x3f, 0xc0, 0x00, 0x00, 0x01, 0xf8, 0x1f, 0xc0, 0x00, 0x00, 0x03, 0xf8, 0x07, 0x80, 0x00, 
	0x00, 0x07, 0xf0, 0x01, 0x80, 0x00, 0x00, 0x07, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 
	0xc0, 0x0f, 0xc0, 0x00, 0x00, 0x0e, 0x70, 0x10, 0x30, 0x00, 0x00, 0x18, 0x18, 0x20, 0x18, 0x00, 
	0x00, 0x20, 0x0c, 0x40, 0x0c, 0x00, 0x00, 0x40, 0x04, 0xc0, 0x04, 0x00, 0x00, 0xc0, 0x02, 0x80, 
	0x06, 0x00, 0x00, 0x80, 0x03, 0x80, 0x02, 0x00, 0x01, 0x80, 0x03, 0x00, 0x03, 0x00, 0x01, 0x00, 
	0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x31, 0x1c, 0x01, 0x00, 
	0x03, 0x00, 0x79, 0x3c, 0x01, 0x00, 0x03, 0x00, 0xf9, 0x3e, 0x01, 0x00, 0x03, 0x00, 0xf9, 0x3e, 
	0x01, 0x00, 0x03, 0x00, 0xf9, 0x3e, 0x01, 0x00, 0x01, 0x00, 0x7b, 0x1c, 0x01, 0x00, 0x01, 0x00, 
	0x33, 0x80, 0x01, 0x00, 0x01, 0x00, 0x02, 0x80, 0x03, 0x00, 0x01, 0x80, 0x04, 0x80, 0x02, 0x00, 
	0x00, 0x80, 0x04, 0x40, 0x06, 0x00, 0x00, 0xc0, 0x08, 0x20, 0x04, 0x00, 0x00, 0x40, 0x18, 0x30, 
	0x08, 0x00, 0x00, 0x30, 0x30, 0x18, 0x10, 0x00, 0x00, 0x1f, 0xc0, 0x07, 0xe0, 0x80, 0x01, 0x07, 
	0x00, 0x00, 0x00, 0x40, 0x03, 0x00, 0x00, 0x00, 0x00, 0x60, 0x06, 0x00, 0x00, 0x00, 0x00, 0x80, 
	0x07, 0x00, 0x00, 0x00, 0x01, 0x80, 0x00, 0xc0, 0x00, 0x00, 0x03, 0x00, 0x00, 0x70, 0x00, 0x00, 
	0x0c, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x38, 0x00, 0x00, 0x07, 0xc0, 0x03, 0xe0, 0x00, 0x00, 0x00, 
	0xff, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
// 'Smiley10', 48x48px
const unsigned char Smiley10_bw [] PROGMEM = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0xf0, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 
	0x00, 0x00, 0x00, 0x03, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x0f, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x1f, 
	0xe0, 0x07, 0xf8, 0x00, 0x00, 0x7f, 0x00, 0x00, 0xfe, 0x00, 0x00, 0xfc, 0x00, 0x00, 0x3f, 0x00, 
	0x01, 0xf8, 0x00, 0x00, 0x1f, 0x80, 0x03, 0xf0, 0x00, 0x00, 0x0f, 0xc0, 0x03, 0xe0, 0x00, 0x00, 
	0x03, 0xc0, 0x07, 0xc0, 0x00, 0x00, 0x03, 0xe0, 0x0f, 0x80, 0x00, 0x00, 0x01, 0xf0, 0x0f, 0x00, 
	0x00, 0x00, 0x00, 0xf0, 0x1f, 0x3c, 0x78, 0x1f, 0x3e, 0x78, 0x1e, 0xfe, 0xfc, 0x3f, 0xff, 0x78, 
	0x1e, 0xff, 0xfe, 0x3f, 0xff, 0x3c, 0x3c, 0xff, 0xfe, 0x7f, 0xff, 0xbc, 0x3c, 0xff, 0xfe, 0x3f, 
	0xff, 0xbc, 0x3c, 0xff, 0xfc, 0x3f, 0xff, 0x1c, 0x78, 0x7f, 0xfc, 0x3f, 0xff, 0x1e, 0x78, 0x7f, 
	0xf8, 0x1f, 0xfe, 0x1e, 0x78, 0x3f, 0xf0, 0x0f, 0xfc, 0x1e, 0x78, 0x1f, 0xf0, 0x07, 0xf8, 0x1e, 
	0x78, 0x0f, 0xe0, 0x03, 0xf0, 0x1e, 0x78, 0x07, 0xc0, 0x01, 0xe0, 0x1e, 0x78, 0x03, 0x00, 0x00, 
	0xc0, 0x1e, 0x78, 0x00, 0x00, 0x00, 0x00, 0x1e, 0x3c, 0x00, 0x00, 0x00, 0x00, 0x1c, 0x3c, 0x00, 
	0x00, 0x00, 0x00, 0x3c, 0x3c, 0x00, 0x00, 0x00, 0x00, 0x3c, 0x1e, 0x0e, 0x00, 0x00, 0x70, 0x3c, 
	0x1e, 0x1f, 0x00, 0x00, 0xf0, 0x78, 0x1f, 0x0f, 0x80, 0x01, 0xf0, 0x78, 0x0f, 0x07, 0xf0, 0x0f, 
	0xe0, 0xf0, 0x0f, 0x81, 0xff, 0xff, 0xc1, 0xf0, 0x07, 0xc0, 0xff, 0xff, 0x03, 0xe0, 0x03, 0xe0, 
	0x3f, 0xfc, 0x07, 0xc0, 0x03, 0xf0, 0x00, 0x00, 0x0f, 0xc0, 0x01, 0xf8, 0x00, 0x00, 0x1f, 0x80, 
	0x00, 0xfe, 0x00, 0x00, 0x7f, 0x00, 0x00, 0x7f, 0x80, 0x01, 0xfe, 0x00, 0x00, 0x1f, 0xf0, 0x0f, 
	0xf8, 0x00, 0x00, 0x0f, 0xff, 0xff, 0xf0, 0x00, 0x00, 0x03, 0xff, 0xff, 0xc0, 0x00, 0x00, 0x00, 
	0x7f, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x07, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
// 'Smiley11', 48x48px
const unsigned char Smiley11_bw [] PROGMEM = {
	0x00, 0x00, 0x07, 0xe0, 0x00, 0x00, 0x00, 0x00, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x03, 0xc0, 0x03, 
	0xc0, 0x00, 0x00, 0x0e, 0x00, 0x00, 0x70, 0x00, 0x00, 0x38, 0x00, 0x00, 0x18, 0x00, 0x00, 0x70, 
	0x00, 0x00, 0x0e, 0x00, 0x00, 0xc0, 0x00, 0x00, 0x03, 0x00, 0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 
	0x03, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x06, 0x00, 0x00, 0x00, 0x00, 0x60, 0x0c, 0x00, 0x00, 0x00, 
	0x00, 0x20, 0x08, 0x00, 0x00, 0x1c, 0x00, 0x30, 0x18, 0x00, 0x00, 0x3e, 0x00, 0x18, 0x10, 0x00, 
	0x00, 0x3f, 0x00, 0x18, 0x30, 0x00, 0x40, 0x3f, 0x00, 0x0c, 0x20, 0x01, 0xf0, 0x3e, 0x00, 0x0c, 
	0x60, 0x01, 0xf8, 0x1c, 0x00, 0x04, 0x60, 0x01, 0xf8, 0x00, 0x00, 0x06, 0x40, 0x01, 0xf0, 0x00, 
	0x00, 0x06, 0x40, 0x00, 0xf0, 0x00, 0x00, 0x02, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x02, 0xc0, 0x00, 
	0x00, 0x00, 0x00, 0x02, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x02, 0xc0, 0x00, 0x00, 0x00, 0x30, 0xc3, 
	0xc0, 0x00, 0x00, 0x00, 0x0f, 0x03, 0xc0, 0x00, 0x00, 0x00, 0x0c, 0x02, 0xc0, 0x00, 0x00, 0x00, 
	0x0c, 0x02, 0xc0, 0x00, 0x40, 0x00, 0x0c, 0x02, 0x40, 0x00, 0x40, 0x00, 0x0c, 0x02, 0x40, 0x00, 
	0x80, 0x00, 0x18, 0x06, 0x60, 0x01, 0x80, 0x00, 0x18, 0x06, 0x60, 0x0e, 0xc0, 0x00, 0x30, 0x04, 
	0x20, 0x00, 0xe0, 0x00, 0x60, 0x0c, 0x30, 0x00, 0x70, 0x01, 0xe0, 0x08, 0x10, 0x00, 0x3c, 0x03, 
	0x80, 0x18, 0x18, 0x00, 0x0f, 0xff, 0x00, 0x10, 0x0c, 0x00, 0x03, 0xfc, 0x00, 0x30, 0x04, 0x00, 
	0x00, 0x00, 0x00, 0x60, 0x06, 0x00, 0x00, 0x00, 0x00, 0x40, 0x03, 0x00, 0x00, 0x00, 0x00, 0xc0, 
	0x01, 0x80, 0x00, 0x00, 0x01, 0x80, 0x00, 0xc0, 0x00, 0x00, 0x07, 0x00, 0x00, 0x70, 0x00, 0x00, 
	0x0c, 0x00, 0x00, 0x18, 0x00, 0x00, 0x38, 0x00, 0x00, 0x0e, 0x00, 0x00, 0xf0, 0x00, 0x00, 0x03, 
	0xc0, 0x07, 0xc0, 0x00, 0x00, 0x00, 0x7f, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x03, 0x80, 0x00, 0x00
};
// 'Smiley12', 48x48px
const unsigned char Smiley12_bw [] PROGMEM = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x78, 0xfc, 0x00, 0x00, 0x00, 0x00, 0xff, 0xfe, 0x00, 0x00, 0x00, 0x01, 0xff, 0xfe, 0x00, 0x00, 
	0x00, 0x01, 0xff, 0xff, 0x00, 0x00, 0x00, 0x03, 0xff, 0xff, 0x00, 0x00, 0x00, 0x03, 0xff, 0xff, 
	0x00, 0x00, 0x3c, 0x03, 0xff, 0xff, 0x80, 0x00, 0x7e, 0x07, 0xff, 0xff, 0x80, 0x00, 0x7e, 0x07, 
	0xff, 0xff, 0x80, 0x1c, 0x7e, 0x07, 0xff, 0xff, 0xc0, 0x3e, 0xfe, 0x07, 0xff, 0xff, 0xc0, 0x7f, 
	0xfe, 0x0f, 0xff, 0xff, 0xc0, 0xff, 0xff, 0x0f, 0xff, 0xff, 0xc1, 0xff, 0xff, 0x0f, 0xff, 0xff, 
	0xc3, 0xff, 0x7f, 0x9f, 0xff, 0xff, 0xc7, 0xff, 0x7f, 0x9f, 0xff, 0xff, 0xcf, 0xff, 0x7f, 0xdf, 
	0xff, 0xff, 0xdf, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xfe, 
	0x1f, 0xff, 0xff, 0xff, 0xbf, 0xfe, 0x1f, 0xe3, 0xff, 0xfe, 0x3f, 0xfc, 0x0f, 0xe0, 0x7f, 0xe0, 
	0x3f, 0xf8, 0x07, 0xe0, 0x70, 0xe0, 0x3f, 0xf8, 0x03, 0xe0, 0x70, 0xe0, 0x1f, 0xf0, 0x00, 0xe0, 
	0x70, 0xe0, 0x1f, 0xe0, 0x00, 0x60, 0x20, 0xe0, 0x1f, 0xc0, 0x00, 0x67, 0x00, 0x0c, 0x3f, 0x80, 
	0x00, 0x63, 0x00, 0x0c, 0x3c, 0x00, 0x00, 0x23, 0x00, 0x1c, 0x20, 0x00, 0x00, 0x21, 0x80, 0x18, 
	0x60, 0x00, 0x00, 0x31, 0x80, 0x30, 0x60, 0x00, 0x00, 0x10, 0xc0, 0x60, 0xc0, 0x00, 0x00, 0x18, 
	0x70, 0xe1, 0xc0, 0x00, 0x00, 0x0c, 0x3f, 0xc1, 0x80, 0x00, 0x00, 0x06, 0x0f, 0x07, 0x00, 0x00, 
	0x00, 0x03, 0x80, 0x0e, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x38, 0x00, 0x00, 0x00, 0x00, 0x3f, 0xf0, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
// 'Version Bildschirm'
const unsigned char Smiley13_bw [] PROGMEM = {
	0x00
};
//}

// ----------------------------------------------------------------------------------------------------------------------------
// globale Variablen
// ----------------------------------------------------------------------------------------------------------------------------
unsigned int fuellWasser, fuellSatz;		// Fuellstaende fuer die Grafik

// Merker fuer verschiedene Halleffekt-Sensoren
bool bTassenSensorTrig = false;
bool bWasserSensorTrig = false;
bool bSatzSensorTrig = false;
// Merker fuer Tasten
bool bKeyRightTrig = false;
bool bKeyLeftTrig = false;
bool bKeyUpTrig = false;
bool bKeyDownTrig = false;
bool bLongKeyRight = false;

// Zaehler der gezapften Tassen
unsigned int iGezapfteTassen = 8600;	// Wert vom mechanischen Tassenzaehler, 04/2018 (5201), 11/2020 (8600)

// aktuelle Werte fuer Wasser- und Satzbehaelter (in Tassen)
unsigned char cWasserZaehler;
unsigned char cSatzZaehler;

// Grenzwerte fuer Wasser- und Satzbehaelter (in Tassen)
unsigned char cWasserGrenze = TASSENADDWASSER;	// wird in Setup aus EEPROM ueberschrieben
unsigned char cSatzGrenze = TASSENREMOVESATZ;	// wird in Setup aus EEPROM ueberschrieben

// Parameter fuer animierte Tasse beim Bezug	// werden in Setup aus EEPROM ueberschrieben
bool bTasseZeigen = false;						// animierte Tasse beim Bezug anzeigen?
bool bErsteRunde = false;
bool bAnimierteTasseAn = false;					// wird true, wenn Tasse gezapft wird
unsigned int iTasseStartZeit = 7;				// warte nach Trigger bis animiertes Fuellen beginnt [s]
unsigned int iTasseFullZeit = 27;				// Zeit von Leer bis Voll [s]
unsigned int iTasseWarteZeit = 9;				// warte nach Fuellen bis Tasse verschwindet [s]
unsigned long lZeilenStepZeit = 0;				// 30 Zeilen fuer volle Tasse, je nach Gesamtdauer des Fuellens
unsigned long lTasseTriggerZeit = 0;			// Zeitpunkt wenn Fuellen beginnt
unsigned long lDampfWechselZeit = 0;			// Timer fuer Dampf-Animation
char cDampf = 0;								// Dampftyp 1 oder 2 -> abwechselnd fuer Animation des Dampfs
unsigned char cAktuelleHelligkeit = 127;		// Displayhelligkeit bei Warnung, 2020-11-26, 2020-11-27
#define UP 1
#define DOWN 0
bool DimDirection = UP;							// Richtung fuer Helligkeitsaenderung beim Dimmen bei Warnungen

// Anzeige-Werte fuer Wasser- und Satz-Grafik (Rechtecke Top und Bottom)
static char cAnzeigeMin = 17;
static char cAnzeigeMax = 63;

char cChangePos = 0;					// welcher Parameter ist im Menue selektiert
										// 0 == Anleitung fuer Aenderungen im Menue
										// 1 == Wasser, 2 == Satz, 3 == Gesamt-Tassenzahl
										// 4 == Params fuer "Tasse zeigen", 5 == ...
int iTouchKey = -1;						// Returnval der TouchKey Abfrage

// Zeiten
unsigned long lTassenZeigeZeit = 0;		// Merker fuer Anzeige der Tassenzahl nach zapfen
unsigned long lZahlenZeigeZeit = 0;		// Merker fuer Anzeige der Tassenzahlen in den Behaeltern
unsigned long lDimDisplayZeit = 0;		// Merker fuer Dimmen der Anzeige, 2020-11-26
unsigned long lWarnPulseZeit = 0;		// Merker fuer pulsierende Warnungen, 2020-11-26
unsigned long lRightKeyPressed = 0;		// Startzeit fuer Auswertung KEYRIGHT lange gedrueckt (> 1s) 
unsigned long lRightKeyLongPressed = 0 ;// Startzeit fuer Auswertung KEYRIGHT lange gedrueckt beenden (> 1s) 
unsigned long lHinweisZeit = 0;			// Hinweis auf Taste RECHTS fuer Menu
unsigned long lMenuTimeout;				// Menue nach Ablauf automatisch wieder verlassen
#define AUTOLEAVEMENU 15000				// Wartezeit fuer Auto-Menu stop (15 Sekunden)
unsigned long lAktivityBlinker = 0;		// Aktivitaetsueberwachung mit LED 13

unsigned long lSmileyPause;				// zufaellig wird anstelle der Behaelter ein Smiley angezeigt
unsigned long lSmileyAnzeige;			// wie lange der Smiley angezeigt wird
bool bSmileySichtbar = false;
unsigned char cSmileySelect;			// welcher Smiley wird angezeigt
unsigned char cSmileyLast = 0;			// welcher Smiley wurde zuletzt angezeigt
bool bCheckLastSmiley = true;			// im Menue nicht auf identische Smileys pruefen
const unsigned char* pSmileyAdresse; 				// die Smileys sind als Array von uint_8 im Flash gespeichert
unsigned char cFrameSelect = 1;			// Rahmen oder Flaeche (zum Einrichten des Display hinter der Frontblende)


#define SMILEYDAMIN 3000				// wie lange wird ein Smiley angezeigt [ms]
#define SMILEYDAMAX 6000
#define SMILEYWEGMIN 10000				// wie lange ist die Pause zwischen zwei Smileys [ms]
#define SMILEYWEGMAX 65500

bool bWarnung = false;					// Flags steuern die Aktualisierung der ersten Zeile

bool inMenu = false;					// Loop durchlaeuft Menue wenn true
bool bFirstTimeInMenu = true;			// Eintritt ins Menue mit KEYRIGHT

// ----------------------------------------------------------------------------------------------------------------------------
// Setup
// ----------------------------------------------------------------------------------------------------------------------------
void setup()   							// Setup um den Arduino zu konfigurieren
{
	wdt_disable();						// Watchdog sicherheitshalber ausschalten
	
	// the value determines how long any key can be in touch before it recalibrates itself. 
	// 160 ms * 62 = 9.92s
	QTouch.setMaxDuration(62);
	// set the threshold value for key0 to register a detection. If the hit pan is bigger, you 
	// should increase this Value. This Value should not be less than 20
	QTouch.setNTHRForKey(60,0);	
	
	// Festlegung der Inputs und Outputs
	pinMode(TASSENSENSOR, INPUT_PULLUP);// Halleffekt-Sensor, OpenCollector, aktiv low
	pinMode(WASSERSENSOR, INPUT_PULLUP);// Halleffekt-Sensor, OpenCollector, aktiv low
	pinMode(SATZSENSOR, INPUT_PULLUP);	// Halleffekt-Sensor, OpenCollector, aktiv low
	pinMode(KEYUP, INPUT_PULLUP);		// Taste nach Masse
	pinMode(KEYDOWN, INPUT_PULLUP);		// Taste nach Masse
	pinMode(KEYLEFT, INPUT_PULLUP);		// Taste nach Masse
	pinMode(KEYRIGHT, INPUT_PULLUP);	// Taste nach Masse
	
	pinMode(GELBELED, OUTPUT);			// nicht angeschlossen, kann zum Debug verwendet werden
	pinMode(LED13, OUTPUT);				// signalisiert OK, blinkt im Sekundentakt
	pinMode(TESTOUT, OUTPUT);			// Testpin zur Bestimmung der Schleifen-Zeit (mit Oszi)
	
	// init OLED display, I2C-Adr. 0x3c, interner StepUp für OLED-Spannung
	display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
	
	// init ADAFRUIT font library
	// "NULL" nutzt den Standard Font (mono spaced, mit deutschen Umlauten)
	// 		display.print(char(132));	// ä
	// 		display.print(char(142));	// Ä
	// 		display.print(char(148));	// ö
	// 		display.print(char(153));	// Ö
	// 		display.print(char(129));	// ü
	// 		display.print(char(154));	// Ü
	// 		display.print(char(225));	// ß (mit cp437-patch, sonst wohl 224)
	// anderenfalls muss der passende font-Header #included und hier angegeben werden
	display.setFont(NULL);
	// char(225) fuer ß stimmt nur, wenn der cp437-Patch angewendet wird
	display.cp437(true);
	
	display.clearDisplay();
	display.setTextColor(WHITE);	// set text color
	display.setTextSize(1);			// set text size

	// Anzahl bisher gezapfter Tassen und andere Zaehler aus EEPROM laden
	readFromEEPROM();	

	// wenn das EEPORM uninitialisiert ist (neuer Chip oder geaenderte Startadresse),
	// steht 0xFF (255 fuer char-Variablen) drin. Dann wird das EEPROM mit Defaults initialisiert.
	// Die in saveToEEPROM() genutzte Funktion prueft, ob der zu speichernde Wert geaendert ist.
	// Nur dann wird ein EEPROM Schreibvorgang durchgefuehrt, spart kostbare Schreibzyklen!
	if((cSatzGrenze == 255) || (cWasserGrenze == 255))
	{
		iGezapfteTassen = 5400;		// Stand 06/2018
		cWasserGrenze = 16;
		cSatzGrenze = 31;
		bTasseZeigen = true;
		iTasseStartZeit = 7;
		iTasseFullZeit = 28;
		iTasseWarteZeit = 9;
		saveToEEPROM();				// und im EEPROM ablegen
	}
	
	randomSeed(iGezapfteTassen);	// Zufallszahlengenerator mit der Tassenzahl initialisieren
	cSmileySelect = random(1, 11);	// einen von 10! Smileys auswaehlen (random(1, 11) erzeugt Zahlen 1..10).
	lSmileyPause = random(SMILEYWEGMIN, SMILEYWEGMAX);	// Smiley kommt alle 10..65 Sekunden
	lSmileyPause += millis(); 		// lSmileyAnzeige = random(1000, 3000);	// Smiley bleibt 1..3 Sekunden sichtbar

	wdt_enable(WDTO_2S);			// Aktiviere Watchdog mit 2s Zeitkonstante. 
									// Muss mindestens 2s sein, damit "Lange RECHTS" nicht zum Reset fuehrt.
}
// ----------------------------------------------------------------------------------------------------------------------------
// Hauptschleife
// ----------------------------------------------------------------------------------------------------------------------------
void loop()
{
	wdt_reset();		// Watchdog retriggern
	
  // Ueberschriften der Grafik ausgeben 
  // bei Warnungen und Menue keine Ueberschrift, Uberschrift immer nur _einmal_ ausgeben
  // ----------------------------------------------------------------------------------------------------------------------------
  if((millis() > (lTassenZeigeZeit + 3000)) && (bWarnung == false) && (inMenu == false))
  {
	display.fillRect(0,0,128,16, BLACK);			// erste Zeile loeschen
	display.setTextColor(WHITE);					// set text color
	display.setTextSize(1);							// set text size
	display.setCursor(TOPLEFTWASSER+4,ZEILE1);		// set text cursor position	
	display.print(F("Wasser"));						// show text
	display.setCursor(TOPLEFTSATZ+8,ZEILE1);		// set text cursor position	
	display.print(F("Satz"));						// show text
	display.display();
  }
  // aktuelle Tassenzahlen alle paar Sekunden anzeigen
  // ----------------------------------------------------------------------------------------------------------------------------
  if((millis() > (lZahlenZeigeZeit + 8000)) && (bWarnung == false) && (inMenu == false))
  {
	display.fillRect(0,0,128,16, BLACK);			// erste Zeile loeschen
	display.setTextColor(WHITE);					// set text color
	display.setTextSize(1);							// set text size
	display.setCursor(TOPLEFTWASSER+16,ZEILE1);		// set text cursor position	
	display.print(cWasserZaehler);					// show text
	display.setCursor(TOPLEFTSATZ+16,ZEILE1);		// set text cursor position	
	display.print(cSatzZaehler);					// show text
	display.display();
	
	lZahlenZeigeZeit = millis();					// Merker setzen fuer Loeschen der Zahlen -> normale Ueberschrift
	lTassenZeigeZeit = millis();					// Merker setzen fuer Loeschen der Tassenzahl -> normale Ueberschrift
  }

  // ----------------------------------------------------------------------------------------------------------------------------
  // nach 2 Minuten OLED dimmen um wear out zu reduzieren, 2020-11-26
  // ----------------------------------------------------------------------------------------------------------------------------
  if((millis() > (lDimDisplayZeit + DIMTIMEOUT))/* && (bWarnung == false) && (inMenu == false)*/)
  {
	setDisplayDim();								// Display dimmen
	lDimDisplayZeit = millis();						// Merker setzen fuer Loeschen der Zahlen -> normale Ueberschrift
  }
  
  digitalWrite(TESTOUT, !digitalRead(TESTOUT));		// Schleifenzeit messen - normal 44 ms -> ~23 Hz

  // ----------------------------------------------------------------------------------------------------------------------------
  // HIER geht die Zapfung los...
  // TASSENSENSOR wird getriggert, wenn eine Tasse gezapft wurde (entspricht dem mechanischen Zaehler)
  // ----------------------------------------------------------------------------------------------------------------------------
  if((digitalRead(TASSENSENSOR) == LOW) && (bTassenSensorTrig == false))		// Magnet ist vorbei gefahren oder liegt permanent an
  {
	setDisplayNorm();								// gedimmtes Display wieder hell machen, 2020-11-26
	iGezapfteTassen++;								// Anzahl Tassen erhoehen
	cWasserZaehler++;								// Zaehler fuer Wasserverbrauch
	cSatzZaehler++;									// Zaehler fuer Kaffeesatzpads
	
	display.fillRect(0,0,128,16, BLACK);			// erste Zeile loeschen
	display.setTextColor(WHITE);					// set text color
	display.setTextSize(1);							// set text size
	display.setCursor(TOPLEFTWASSER+4,ZEILE1);			// set text cursor position	
	display.print(F("Tassenzahl "));				// show text
	display.print(iGezapfteTassen);
	
	// Direkt nach Ausloesen des Tassentriggers wird die leere Kaffeetasse angezeigt (parametrierbar)
	bAnimierteTasseAn = true;						// ab jetzt Tasse anzeigen
	bErsteRunde = true;								// das Display wird fuer die Tasse eimalig geloescht
	lTasseTriggerZeit = millis();					// ab jetzt laufen die AnimierteTasse-Zeiten
	
	// 3 Sekunden Tassenzahl anzeigen
	lTassenZeigeZeit = millis();					// Merker setzen fuer Loeschen der Tassenzahl -> normale Ueberschrift
	
	bTassenSensorTrig = true;						// merken, dass getriggert wurde
	
	saveToEEPROM();									// Werte im EEPROM permanent machen
	
	// digitalWrite(GELBELED, !digitalRead(GELBELED));
  }
  else if((bTassenSensorTrig == true) && (digitalRead(TASSENSENSOR) == HIGH))
  {
	bTassenSensorTrig = false;						// Trigger loeschen
  }

  if(bTassenSensorTrig == true) lDimDisplayZeit = millis(); // Display hell halten solange Kaffe laeuft, 2020-11-26

  // WASSERSENSOR wird getriggert, wenn der Wasserbehaelter herausgezogen wird
  // ----------------------------------------------------------------------------------------------------------------------------
  if((digitalRead(WASSERSENSOR) == LOW) && (bWasserSensorTrig == false))		// Magnet ist vorbei gefahren oder liegt permanent an
  {
	setDisplayNorm();								// gedimmtes Display wieder hell machen, 2020-11-26
	// display.clearDisplay();
	display.fillRect(0,0,128,16, BLACK);			// erste Zeile loeschen
	display.fillRect(0,16,128,48, BLACK);			// unteres Display loeschen // top-left X, top-left Y, Width, Height
	display.setTextColor(WHITE);					// set text color
	display.setTextSize(1);							// set text size
	display.setCursor(TOPLEFTWASSER+4,ZEILE1);			// set text cursor position	
	display.print(F("Wasser getriggert"));
	display.display();

	// 3 Sekunden Werte anzeigen
	lTassenZeigeZeit = millis();					// Merker setzen fuer Loeschen der Werte -> normale Ueberschrift

	cWasserZaehler = 0;			// Tassenzaehler reset da Wasserbehaelter rausgezogen wurde == Nachfuellen
	bWasserSensorTrig = true;						// merken, dass getriggert wurde
	
	saveToEEPROM();									// Werte im EEPROM permanent machen

	// digitalWrite(GELBELED, !digitalRead(GELBELED));
  }
  else if((bWasserSensorTrig == true) && (digitalRead(WASSERSENSOR) == HIGH))
  {
	bWasserSensorTrig = false;						// Trigger loeschen
  }
  
  // SATZSENSOR wird getriggert, wenn der Satzbehaelter herausgezogen wird
  // ----------------------------------------------------------------------------------------------------------------------------
  if((digitalRead(SATZSENSOR) == LOW) && (bSatzSensorTrig == false))		// Magnet ist vorbei gefahren oder liegt permanent an
  {
	setDisplayNorm();								// gedimmtes Display wieder hell machen, 2020-11-26
	// display.clearDisplay();
	display.fillRect(0,0,128,16, BLACK);			// erste Zeile loeschen
	display.fillRect(0,16,128,48, BLACK);			// unteres Display loeschen // top-left X, top-left Y, Width, Height
	display.setTextColor(WHITE);					// set text color
	display.setTextSize(1);							// set text size
	display.setCursor(TOPLEFTWASSER+4,ZEILE1);		// set text cursor position	
	display.print(F("Satz getriggert"));
	// display.print(F("123456789012345678901 234567890"));	// 21 Zeichen mit TextSize 1
	display.display();

	// 3 Sekunden Werte anzeigen
	lTassenZeigeZeit = millis();					// Merker setzen fuer Loeschen der Werte -> normale Ueberschrift

	cSatzZaehler = 0;			// Tassenzaehler reset da Satzbehaelter rausgezogen wurde == Leeren
	bSatzSensorTrig = true;							// merken, dass getriggert wurde
	
	saveToEEPROM();									// Werte im EEPROM permanent machen
	
	// digitalWrite(GELBELED, !digitalRead(GELBELED));	
  }
  else if((bSatzSensorTrig == true) && (digitalRead(SATZSENSOR) == HIGH))
  {
	bSatzSensorTrig = false;						// Trigger loeschen
  }
  
  // Tastendruck RECHTS fuehrt ins Menue
  // ----------------------------------------------------------------------------------------------------------------------------
  if(GETKEYMACRO(QTRIGHT)) inMenu = true;	
  
  if (!inMenu) 					// normaler Betrieb
  {
	// display.clearDisplay();
	// um unterschiedliche Bereiche des Display loeschen zu koennen verwende ich schwarze Rechtecke
	// display.fillRect(0,16,128,48, BLACK);		// unteres Display loeschen // top-left X, top-left Y, Width, Height

	// ----------------------------------------------------------------------------------------------------------------------------
	// Logik fuer animierte Tasse beim Bezug
	// ----------------------------------------------------------------------------------------------------------------------------
	// Direkt nach Ausloesen des Tassentriggers wird die leere Kaffeetasse angezeigt (parametrierbar)
	if(bTasseZeigen == true)							// im Menue ist "Animierte Tasse" eingeschaltet
	{
		if(bAnimierteTasseAn == true)					// ein Zapfvorgang wurde gestartet
		{
			if(bErsteRunde == true)						// nur einmal das Display loeschen
			{
				display.fillRect(0,16,128,48, BLACK);	// unteres Display loeschen // top-left X, top-left Y, Width, Height
				bErsteRunde = false;
				iLineCounter = 0;						// bei neuem Bezug Tasse von unten fuellen
			}
			// void drawBitmap(int16_t x, int16_t y, uint8_t *bitmap, int16_t width, int16_t height, uint16_t color);
			display.drawBitmap(30, 30,  KaffeeTasse, 64, 36, 1);
			// display.display();
		
			if(millis() < (lTasseTriggerZeit + (unsigned long)(iTasseStartZeit * 1000)))
			{
				// zeige die leere Tasse und warte auf den Start des Bezugs
			}
			else if(millis() < (lTasseTriggerZeit + (unsigned long)(iTasseStartZeit * 1000) + (unsigned long)(iTasseFullZeit * 1000)))
			{
				// Bezug ist gestartet, fuelle die Tasse von 0..100% innerhalb der parametrierten Zeit
				// iTasseFullZeit ist 100%
				// virtual void writeFastHLine(int16_t x, int16_t y, int16_t w, uint16_t color);
				if(millis() > (lZeilenStepZeit + ((float)((float)iTasseFullZeit / (float)TASSEZEILEN)) * 1000))
				{
					// digitalWrite(GELBELED, HIGH);			// Debug
					unsigned char cIndex = iLineCounter * 3;
					// (cIndex > 9 ? 
					display.writeFastHLine(pgm_read_word(&FuellLinien[cIndex]), pgm_read_word(&FuellLinien[cIndex + 1]), pgm_read_word(&FuellLinien[cIndex + 2]), WHITE);	//:
					// display.writeFastHLine(FuellLinien[9], FuellLinien[9 + 1] - (iLineCounter - 9), FuellLinien[9 + 2], WHITE);
					iLineCounter++;
					if(iLineCounter > 26) iLineCounter = 26;	// Sicherung gegen "out of bounds" error
					lZeilenStepZeit = millis();
				}
			}
			else if(millis() < (lTasseTriggerZeit + (unsigned long)(iTasseStartZeit * 1000) + (unsigned long)(iTasseFullZeit * 1000) + (unsigned long)(iTasseWarteZeit * 1000)))
			{
				// digitalWrite(GELBELED, LOW);			// Debug
				// Dampf-Animation anzeigen
				if(millis() > lDampfWechselZeit + 1000)
				{
					display.fillRect(50,16,16,15, BLACK); 			// top-left X, top-left Y, Width, Height
					if(cDampf == 0)
					{
						display.drawBitmap(50, 16,  Dampf1, 16, 16, WHITE);
						cDampf = 1;									// Dampftyp 2 
					}
					else
					{
						// display.drawRect(52,16,16,15, BLACK); 	// top-left X, top-left Y, Width, Height
						display.drawBitmap(50, 16,  Dampf2, 16, 16, WHITE);
						cDampf = 0;									// Dampftyp 1
					}
					lDampfWechselZeit = millis();
				}
			}
			else
			{
				bAnimierteTasseAn = false;				// nach Ablauf aller Anzeigezeiten Tasse loeschen
			}
		}
		else
		{
			// Warnungen oder Fuellstaende anzeigen (wenn "animierte Tasse" im Menue eingeschaltet ist)
			if((cWasserZaehler > cWasserGrenze-1) || (cSatzZaehler > cSatzGrenze-1))
				displayWarnungen(); else displayFuellStaende();
		}
	}
	else
	{
		// Warnungen oder Fuellstaende anzeigen (wenn "animierte Tasse" im Menue ausgeschaltet ist)
		if((cWasserZaehler > cWasserGrenze-1) || (cSatzZaehler > cSatzGrenze-1))
			displayWarnungen(); else displayFuellStaende();
	}

 	display.display();								// alle Display-Aenderungen anzeigen - Display aktualisieren

  }	// 	\nicht im Menue
  else												// if(!inMenu)
  {	
  // ab hier _im_ Menue ---------------------------------------------------------------------------------------------------------
  // ----------------------------------------------------------------------------------------------------------------------------
	setDisplayNorm();								// gedimmtes Display wieder hell machen, 2020-11-26

	if((GETKEYMACRO(QTRIGHT)) && (bFirstTimeInMenu == true))		// Abfrage mit QTouch
	{
		cChangePos = 0;								// beim Eintritt ins Menue einmalig Anleitung anzeigen
		bFirstTimeInMenu = false;
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden
		
		while(GETKEYMACRO(QTRIGHT)) {};				// hier warten, bis RIGHTKEY wieder losgelassen wurde
	}

  // ----------------------------------------------------------------------------------------------------------------------------
  // RIGHTKEY lange festhalten (>1 s, aber <2 s wegen Watchdog)
	if((GETKEYMACRO(QTRIGHT)) && (bKeyRightTrig == false) && (bLongKeyRight == false))
	{
		lRightKeyPressed = millis();				// Startzeit des Tastendrucks merken
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden
		if(cChangePos == 0) (cChangePos = 1);		// wechselt von 0 auf 1 - bei 0 -> Anleitung anzeigen
		else if(cChangePos == 1) (cChangePos = 2);	// wechselt zwischen 1..9 -> dort Werte aendern
		else if(cChangePos == 2) (cChangePos = 3);
		else if(cChangePos == 3) (cChangePos = 4);
		else if(cChangePos == 4) (cChangePos = 5);
		else if(cChangePos == 5) (cChangePos = 6);
		else if(cChangePos == 6) (cChangePos = 7);
		else if(cChangePos == 7) (cChangePos = 8);
		else if(cChangePos == 8) (cChangePos = 9);
		// else if(cChangePos == 9) (cChangePos = 10);		// cChangePos == 9 verlassen durch "lange Rechts"
		else if(cChangePos == 10) (cChangePos = 1);
		
		while((GETKEYMACRO(QTRIGHT)) && (bLongKeyRight == false))	
		{	
			if(millis() > (lRightKeyPressed + 1000))// RIGHTKEY laenger als 1s gedrueckt?
			{										//  -> "kontinuierlich" fuer Gesamt-Tassenzahl einschalten
				if(cChangePos == 9)					// aus cChangePos == 9 (Anzeige der QT-Werte) kommt man mit 
				{									// "lange Rechts" wieder raus
					(cChangePos = 10);
					// lRightKeyPressed = millis();	// neue Startzeit des Tastendrucks merken
					while((GETKEYMACRO(QTRIGHT)));	// hier warten, bis RIGHT wieder losgelassen wurden
				}									
				else
				{
					bLongKeyRight = true;
					cChangePos = 3;					// kontinuierlich aendern nur fuer Gesamt-Tassenzahl
				}
			}
		}		
		bKeyRightTrig = true;						// merken, dass getriggert wurde
	}
	else if((bKeyRightTrig == true) && (!GETKEYMACRO(QTRIGHT)))
	{
		bKeyRightTrig = false;						// Trigger loeschen
	}
	
  // ----------------------------------------------------------------------------------------------------------------------------
    // Status "RIGHTKEY lange gedrueckt" wieder abschalten
	if((GETKEYMACRO(QTRIGHT)) && (bKeyRightTrig == false) && (bLongKeyRight == true))
	{
		lRightKeyLongPressed = millis();									// Startzeit des Tastendrucks merken
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden
		while((GETKEYMACRO(QTRIGHT)) && (bLongKeyRight == true))			// RIGHT laenger als 1s gedrueckt?
		{																	//  -> "kontinuierlich" fuer Gesamt-Tassenzahl ausschalten
			if(millis() > (lRightKeyLongPressed + 1000))
			{
				bLongKeyRight = false;
			}
		}		
		bKeyRightTrig = true;												// merken, dass getriggert wurde
	}
	else if((bKeyRightTrig == true) && (!GETKEYMACRO(QTRIGHT)) && (bLongKeyRight == true))
	{
		bKeyRightTrig = false;												// Trigger loeschen
	}
	
	// Tassenwerte Wasser, Satz oder Gesamttassenzahl schrittweise erhoehen
  // ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTUP)) &&  (bKeyUpTrig == false) && (bLongKeyRight == false))
	{
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden
		if(cChangePos == 1)										// Tassen bis Wasser nachgefuellt werden muss
		{
			cWasserGrenze = cWasserGrenze + 1;
			cWasserGrenze = constrain(cWasserGrenze, 12, 18);	// nur Werte zwischen 12 und 18 sind zulaessig
		}
		else if(cChangePos == 2)								// Tassen bis Satzbehaelter geleert werden muss
		{
			cSatzGrenze = cSatzGrenze + 1;
			cSatzGrenze = constrain(cSatzGrenze, 25, 35);		// nur Werte zwischen 25 und 35 sind zulaessig
		}
		else if(cChangePos == 3)								// Gesamt-Tassenzahl
		{
			iGezapfteTassen = iGezapfteTassen + 1;
			iGezapfteTassen = constrain(iGezapfteTassen, 5000, 65535);	// nur Werte zwischen 5000 und 65535 sind zulaessig
		}
		else if(cChangePos == 4)								// animierte Tasse anzeigen?
		{
			if(bTasseZeigen == true) bTasseZeigen = false; else bTasseZeigen = true; 
		}
		else if(cChangePos == 5)								// Zeit[s] nach Trigger bis Tasse anfaengt sich zu fuellen
		{
			iTasseStartZeit = iTasseStartZeit + 1;
			iTasseStartZeit = constrain(iTasseStartZeit, 1, 30);	// nur Werte zwischen 1 und 30 sind zulaessig
		}
		else if(cChangePos == 6)								// Zeit[s] nach Startzeit bis Tasse voll ist
		{
			iTasseFullZeit = iTasseFullZeit + 1;
			iTasseFullZeit = constrain(iTasseFullZeit, 10, 59);	// nur Werte zwischen 10 und 59 sind zulaessig
		}
		else if(cChangePos == 7)								// Zeit[s] nach FullZeit in der Dampf angezeigt wird
		{
			iTasseWarteZeit = iTasseWarteZeit + 1;
			iTasseWarteZeit = constrain(iTasseWarteZeit, 1, 9);	// nur Werte zwischen 1 und 9 sind zulaessig
		}
		else if(cChangePos == 8)								// alle Smileys werden reihum angezeigt
		{
			cSmileySelect = cSmileySelect + 1;
			if(cSmileySelect > 11) cSmileySelect = 1;			// wrap around, 2020-11-27
			// cSmileySelect = constrain(cSmileySelect, 1, 10);	// nur Werte zwischen 1 und 10 sind zulaessig
		}
		else if(cChangePos == 10)								// Flaeche oder Rahmen zum Zentrieren des Display in der Blende
		{
			if(cFrameSelect == 1) cFrameSelect = 2; else cFrameSelect = 1;
		}
		bKeyUpTrig = true;							// merken, dass getriggert wurde
	}		
	else if((bKeyUpTrig == true) && (!GETKEYMACRO(QTUP)))
	{
		bKeyUpTrig = false;							// Trigger loeschen
	}
	
	// Tassenwert Wasser, Satz oder Gesamttassenzahl schrittweise vermindern
  // ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTDOWN)) &&  (bKeyDownTrig == false) && (bLongKeyRight == false))
	{
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden
		if(cChangePos == 1)
		{
			cWasserGrenze = cWasserGrenze - 1;
#ifdef TESTMODE
 			cWasserGrenze = constrain(cWasserGrenze, 5, 8);		// nur Werte zwischen 12 und 18 sind zulaessig
#else
			cWasserGrenze = constrain(cWasserGrenze, 12, 18);	// nur Werte zwischen 12 und 18 sind zulaessig
#endif
		}
		else if(cChangePos == 2)
		{
			cSatzGrenze = cSatzGrenze - 1;
#ifdef TESTMODE
			cSatzGrenze = constrain(cSatzGrenze, 8, 12);		// nur Werte zwischen 25 und 35 sind zulaessig
#else
			cSatzGrenze = constrain(cSatzGrenze, 25, 35);		// nur Werte zwischen 25 und 35 sind zulaessig
#endif
		}
		else if(cChangePos == 3)
		{
			iGezapfteTassen = iGezapfteTassen - 1;
			iGezapfteTassen = constrain(iGezapfteTassen, 5000, 65535);	// nur Werte zwischen 5000 und 65535 sind zulaessig
		}
		else if(cChangePos == 4)
		{
			if(bTasseZeigen == true) bTasseZeigen = false; else bTasseZeigen = true; 
		}
		else if(cChangePos == 5)
		{
			iTasseStartZeit = iTasseStartZeit - 1;
			iTasseStartZeit = constrain(iTasseStartZeit, 1, 30);	// nur Werte zwischen 1 und 30 sind zulaessig
		}
		else if(cChangePos == 6)
		{
			iTasseFullZeit = iTasseFullZeit - 1;
#ifdef TESTMODE
			iTasseFullZeit = constrain(iTasseFullZeit, 3, 35);	// nur Werte zwischen 10 und 59 sind zulaessig
#else
			iTasseFullZeit = constrain(iTasseFullZeit, 10, 59);	// nur Werte zwischen 10 und 59 sind zulaessig
#endif
		}
		else if(cChangePos == 7)
		{
			iTasseWarteZeit = iTasseWarteZeit - 1;
			iTasseWarteZeit = constrain(iTasseWarteZeit, 1, 9);	// nur Werte zwischen 1 und 9 sind zulaessig
		}
		else if(cChangePos == 8)
		{
			cSmileySelect = cSmileySelect - 1;
			if(cSmileySelect < 1) cSmileySelect = 11;			// wrap around, 2020-11-27
			// cSmileySelect = constrain(cSmileySelect, 1, 10);	// nur Werte zwischen 1 und 10 sind zulaessig
		}
		else if(cChangePos == 10)
		{
			if(cFrameSelect == 1) cFrameSelect = 2; else cFrameSelect = 1;
		}
		bKeyDownTrig = true;						// merken, dass getriggert wurde
	}		
	else if((bKeyDownTrig == true) && (!GETKEYMACRO(QTDOWN)))
	{
		bKeyDownTrig = false;						// Trigger loeschen
	}
	
	// Gesamttassenzahl kontinuierlich erhoehen - nur nach langem Tastendruck RECHTS
  // ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTUP)) &&  (bKeyUpTrig == false) && (bLongKeyRight == true))
	{
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden
		iGezapfteTassen = iGezapfteTassen + 1;
		iGezapfteTassen = constrain(iGezapfteTassen, 5000, 65535);	// nur Werte zwischen 5000 und 65535 sind zulaessig
	}		

	// Gesamttassenzahl kontinuierlich vermindern - nur nach langem Tastendruck RECHTS
  // ----------------------------------------------------------------------------------------------------------------------------
	if((GETKEYMACRO(QTDOWN)) &&  (bKeyDownTrig == false) && (bLongKeyRight == true))
	{
		lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
													// der Timer muss bei jedem Tastendruck nachgetriggert werden
		iGezapfteTassen = iGezapfteTassen - 1;
		iGezapfteTassen = constrain(iGezapfteTassen, 5000, 65535);	// nur Werte zwischen 5000 und 65535 sind zulaessig
	}		

  // ----------------------------------------------------------------------------------------------------------------------------
  // Hier werden die statischen Elemente der Texte im Menue dargestellt (abhaengig von cChangePos)
  // ----------------------------------------------------------------------------------------------------------------------------
	if(cChangePos == 0)								// Anleitung fuer das Menue
	{
		display.clearDisplay();
		display.setTextSize(1);						// set text size
		display.setTextColor(WHITE);				// set text color
		display.setCursor(TOPLEFTWASSER+4,ZEILE1);	// set text cursor position	("-16" == 1. Zeile, gelb)
		display.println(F("Einstellungen"));		// show text
		
		display.setCursor(TEXTLEFT,ZEILE2);			// set text cursor position	("-2" == 2. Zeile)
		display.println(F("Rechts: Auswahl"));		// show text
		display.setCursor(TEXTLEFT,ZEILE3-1);		// set text cursor position	("+9" == 3. Zeile)
		display.println(F("Rechts >1 Sekunde:"));	// show text
		display.setCursor(TEXTLEFT,TEXTTOP+17);		// set text cursor position	("+17" == 4. Zeile)
		display.print(F("Kontinuierlich "));
		display.print(char(132));					// ä
		display.println(F("ndern"));				// show text
		display.setCursor(TEXTLEFT,TEXTTOP+28);		// set text cursor position	("+28" == 5. Zeile)
		display.print(F("Hoch/Runter: "));
		display.print(char(142));					// Ä
		display.println(F("ndern"));				// show text
		display.setCursor(TEXTLEFT,TEXTTOP+39);		// set text cursor position	("+39" == 6. Zeile)
		display.println(F("Links: Verlassen"));		// show text
	}
	else if((cChangePos == 1) || (cChangePos == 2) || (cChangePos == 3))
	{
		// Menue-Text fuer Tassenvorgaben
		display.clearDisplay();
		display.setTextSize(1);						// set text size
		display.setTextColor(WHITE);				// set text color
		display.setCursor(TOPLEFTWASSER+4,ZEILE1);	// set text cursor position	("-16" == 1. Zeile)
		display.println(F("Einstellungen"));		// show text
		
		display.setCursor(TEXTLEFT,ZEILE2);			// set text cursor position	("-2" == 2. Zeile)
		display.print(F("Tassen f"));
		display.print(char(129));					// ü
		display.print(F("r Beh"));
		display.print(char(132));					// ä
		display.println(F("lter"));					// show text
		display.setCursor(TEXTLEFT,ZEILE3);			// set text cursor position	("+10" == ~3. Zeile)
		display.println(F("Wasser:    Satz:   "));	// show text
		display.setCursor(TEXTLEFT,ZEILE4);			// set text cursor position	("+22" == ~5. Zeile)
		display.println(F("Gezapfte Tassen:"));		// show text
	}
	else if((cChangePos == 4) || (cChangePos == 5) || (cChangePos == 6))
	{
		// Parameter fuer Tassen-Grafik einstellen
		display.clearDisplay();
		display.setTextSize(1);						// set text size
		display.setTextColor(WHITE);				// set text color
		display.setCursor(TOPLEFTWASSER+4,ZEILE1);	// set text cursor position	("-16" == 1. Zeile)
		display.print(F("Einstellungen"));			// show text
		
		display.setCursor(TEXTLEFT,ZEILE2);			// set text cursor position	("-2" == 2. Zeile)
		display.print(F("Animierte Tasse"));		// show text
		display.setCursor(TEXTLEFT+94,ZEILE2);		// set text cursor position
		display.print((bTasseZeigen == true ? " Ja " : "Nein"));
		
		display.setCursor(TEXTLEFT,ZEILE3);			// set text cursor position	("+10" == 3. Zeile)
		display.print(F("Zeit bis Start:    s"));

		display.setCursor(TEXTLEFT,ZEILE4);			// set text cursor position	("+22" == 4. Zeile)
		display.print(F("Zeit bis Voll :    s"));

		display.setCursor(TEXTLEFT,ZEILE5);			// set text cursor position	("+34" == 5. Zeile)
		display.print(F("Zeit nach Voll:    s"));
	}
	else if((cChangePos == 8) || (cChangePos == 9)) // Smileys oder QTouch-Werte darstellen
	{
		display.clearDisplay();
		display.setTextSize(1);						// set text size
		display.setTextColor(WHITE);				// set text color
		display.setCursor(TOPLEFTWASSER+4,ZEILE1);	// set text cursor position	("-16" == 1. Zeile)
		display.print(F("Einstellungen"));			// show text
	}
	else if(cChangePos == 10) 						// Rahmen zum Ausrichten des Display darstellen
	{
		display.clearDisplay();
	}
	
  // ----------------------------------------------------------------------------------------------------------------------------
  // Hier werden die dynamischen Elemente der Texte im Menue dargestellt
  // ----------------------------------------------------------------------------------------------------------------------------
	if(cChangePos == 1)							// Die zu aendernde Stelle (Wasser, Satz oder Tassen) wird invers dargestellt
	{
		display.fillRect(TEXTLEFT+46,ZEILE3-1, 13, 9, WHITE);		// ---> Wasser selektieren <---
		display.setTextSize(1);										// set text size
		display.setTextColor(BLACK);								// set text color
		display.setCursor(TEXTLEFT+47,ZEILE3);						// set text cursor position
		display.print(cWasserGrenze);
		// --------------------------
		display.fillRect(TEXTLEFT+100,ZEILE3-1, 13, 9, BLACK);		// Satz deselektieren
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+101,ZEILE3);						// set text cursor position
		display.print(cSatzGrenze);
		// --------------------------
		display.fillRect(TEXTLEFT+46,ZEILE5-1, 25, 9, BLACK);		// Gesamttassenzahl deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+47,ZEILE5);						// set text cursor position
		display.print(iGezapfteTassen);
	}
	else if(cChangePos == 2)
	{
		display.fillRect(TEXTLEFT+46,ZEILE3-1, 13, 9, BLACK);		// Wasser deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+47,ZEILE3);						// set text cursor position
		display.print(cWasserGrenze);
		// --------------------------
		display.fillRect(TEXTLEFT+100,ZEILE3-1, 13, 9, WHITE);		// ---> Satz selektieren <---
		display.setTextColor(BLACK);								// set text color
		display.setCursor(TEXTLEFT+101,ZEILE3);						// set text cursor position
		display.print(cSatzGrenze);
		// --------------------------
		display.fillRect(TEXTLEFT+46,ZEILE5-1, 25, 9, BLACK);		// Gesamttassenzahl deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+47,ZEILE5);						// set text cursor position
		display.print(iGezapfteTassen);
	}
	else if(cChangePos == 3)
	{
		display.fillRect(TEXTLEFT+46,ZEILE3-1, 13, 9, BLACK);		// Wasser deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+47,ZEILE3);						// set text cursor position
		display.print(cWasserGrenze);
		// --------------------------
		display.fillRect(TEXTLEFT+100,ZEILE3-1, 13, 9, BLACK);		// Satz deselektieren
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+101,ZEILE3);						// set text cursor position
		display.print(cSatzGrenze);
		// --------------------------
		display.fillRect(TEXTLEFT+46,ZEILE5-1, 25, 9, WHITE);		// ---> Gesamttassenzahl selektieren <---
		display.setTextSize(1);										// set text size
		display.setTextColor(BLACK);								// set text color
		display.setCursor(TEXTLEFT+47,ZEILE5);						// set text cursor position
		display.print(iGezapfteTassen);
		
		if(bLongKeyRight == true)
		{
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+10,ZEILE5);						// set text cursor position
		display.print("->");
		display.setCursor(TEXTLEFT+96,ZEILE5);						// set text cursor position
		display.print("<-");
		}
	}
	else if(cChangePos == 4)
	{
		display.fillRect(TEXTLEFT+93,ZEILE2, 26, 9, WHITE);			// Ja/Nein selektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(BLACK);								// set text color
		display.setCursor(TEXTLEFT+94,ZEILE2);						// set text cursor position
		display.print((bTasseZeigen == true ? " Ja " : "Nein"));
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE3-1, 13, 9, BLACK);		// Startzeit deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+93,ZEILE3);						// set text cursor position
		display.print(iTasseStartZeit < 10 ? " " : "");
		display.print(iTasseStartZeit);
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE4-1, 13, 9, BLACK);		// Fuellzeit deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+93,ZEILE4);						// set text cursor position
		display.print(iTasseFullZeit);
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE5-1, 13, 9, BLACK);		// Wartezeit deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+99,ZEILE5);						// set text cursor position
		display.print(iTasseWarteZeit);
	}
	else if(cChangePos == 5)
	{
		display.fillRect(TEXTLEFT+93,ZEILE2, 26, 9, BLACK);			// Ja/Nein deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+94,ZEILE2);						// set text cursor position
		display.print((bTasseZeigen == true ? " Ja " : "Nein"));
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE3-1, 13, 9, WHITE);		// Startzeit selektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(BLACK);								// set text color
		display.setCursor(TEXTLEFT+93,ZEILE3);						// set text cursor position
		display.print(iTasseStartZeit < 10 ? " " : "");
		display.print(iTasseStartZeit);
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE4-1, 13, 9, BLACK);		// Fuellzeit deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+93,ZEILE4);						// set text cursor position
		display.print(iTasseFullZeit);
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE5-1, 13, 9, BLACK);		// Wartezeit deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+99,ZEILE5);						// set text cursor position
		display.print(iTasseWarteZeit);
	}
	else if(cChangePos == 6)
	{
		display.fillRect(TEXTLEFT+93,ZEILE2, 26, 9, BLACK);			// Ja/Nein deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+94,ZEILE2);						// set text cursor position
		display.print((bTasseZeigen == true ? " Ja " : "Nein"));
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE3-1, 13, 9, BLACK);		// Startzeit deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+93,ZEILE3);						// set text cursor position
		display.print(iTasseStartZeit < 10 ? " " : "");
		display.print(iTasseStartZeit);
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE4-1, 13, 9, WHITE);		// Fuellzeit selektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(BLACK);								// set text color
		display.setCursor(TEXTLEFT+93,ZEILE4);						// set text cursor position
		display.print(iTasseFullZeit);
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE5-1, 13, 9, BLACK);		// Wartezeit deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+99,ZEILE5);						// set text cursor position
		display.print(iTasseWarteZeit);
	}
	else if(cChangePos == 7)
	{
		display.fillRect(TEXTLEFT+93,ZEILE2, 26, 9, BLACK);			// Ja/Nein deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+94,ZEILE2);						// set text cursor position
		display.print((bTasseZeigen == true ? " Ja " : "Nein"));
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE3-1, 13, 9, BLACK);		// Startzeit deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+93,ZEILE3);						// set text cursor position
		display.print(iTasseStartZeit < 10 ? " " : "");
		display.print(iTasseStartZeit);
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE4-1, 13, 9, BLACK);		// Fuellzeit deselektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(WHITE);								// set text color
		display.setCursor(TEXTLEFT+93,ZEILE4);						// set text cursor position
		display.print(iTasseFullZeit);
		// --------------------------
		display.fillRect(TEXTLEFT+92,ZEILE5-1, 13, 9, WHITE);		// Wartezeit selektieren
		display.setTextSize(1);										// set text size
		display.setTextColor(BLACK);								// set text color
		display.setCursor(TEXTLEFT+99,ZEILE5);						// set text cursor position
		display.print(iTasseWarteZeit);
	}
	else if(cChangePos == 8)										// reihum alle Smileys ausgeben
	{
		bCheckLastSmiley = false;									// im Menue kein Check auf gleiche Smileys
		displaySmiley();
	}
	else if(cChangePos == 9)										// QTouch-Werte der Tasten ausgeben
	{
	#ifdef QTOUCHKEYS												// nur wenn QTouchInput aktiviert ist, 2020-11-27
		showQTvalues();
	#else															// wenn normale Tasten oder Einzelsensoren...
		cChangePos++;												// ...eingesetzt werden, überspringen
	#endif
	}
	else if(cChangePos == 10)										// Rahmen/Fläche anzeigen zum Ausrichten des Display im Deckel
	{
		showFrames();
	}
	
	display.display();
	
	// Menue verlassen (hier muss keine Verriegelung programmiert werden)
  // ----------------------------------------------------------------------------------------------------------------------------
  // Das Menu wird verlassen wenn LINKS betaetigt wird oder wenn der Timeout abgelaufen ist, ohne dass 
  // eine Taste betaetigt wurde.
  // Im Menue "cChangePos == 9" (QTouch-Werte anzeigen) ist LINKS wirkungslos, auch dessen Wert soll 
  // angezeigt werden, ohne das Menue dabei zu verlassen.
  	if(((GETKEYMACRO(QTLEFT)) || (millis() > (lMenuTimeout + AUTOLEAVEMENU))) && (cChangePos != 9))
	{
		// display.clearDisplay();
		inMenu = false;					// Menue verlassen
		bLongKeyRight = false;			// kontinuierlichen Modus loeschen
		
		// Werte im EEPROM permanent machen, nur geaenderte Werte werden dabei geschrieben
		saveToEEPROM();					
		
		bFirstTimeInMenu = true;		// beim Verlassen des Menu wieder setzen
		lDimDisplayZeit = millis();		// beim Verlassen des Menu den Timer neu starten (sonst waere sofort DIM), 2020-11-26
	}
  }	// \im Menue
  
  // Aktivitaetsueberwachung. Beim Bezug einer Tasse im Automaten eingebaut ist das Display mehrfach eingefroren.
  // Hiermit ist feststellbar, ob die Kommunikation zum Display (I2C) zusammengebrochen ist, oder der µC haengt...
  if(millis() > (lAktivityBlinker + 1000)) 
  {
	  digitalWrite(LED13, !digitalRead(LED13));		// LED13 blinkt im Sekundentakt
	  lAktivityBlinker = millis();
  }

}	// \loop()
// ----------------------------------------------------------------------------------------------------------------------------

// ----------------------------------------------------------------------------------------------------------------------------
// Unterprogramme 
// ----------------------------------------------------------------------------------------------------------------------------
// Variablen ins EEPROM schreiben (nur wenn geaendert, das macht ".put" automatisch so)
// ----------------------------------------------------------------------------------------------------------------------------
void saveToEEPROM()
{	// alle Werte werden immer auf einmal behandelt, EEPROM.put() regelt, ob
	// der jeweilige Wert tatsächlich geschrieben werden muss (weil geaendert)
	// Damit das funktioniert, muss save und read immer in der gleichen Reihenfolge passieren
	int eeAddress = EESTARTADRESSE;	// immmer auf Adresse 0 starten (waehrend Entwicklung: Adresse 1000)
	
	EEPROM.put(eeAddress, iGezapfteTassen);
	eeAddress += sizeof(int); 			//Move address to the next byte after int 'i' (2 Byte)
	EEPROM.put(eeAddress, cWasserZaehler);
	eeAddress += sizeof(unsigned char); //Move address to the next byte after unsigned char 'c' (1 Byte)
	EEPROM.put(eeAddress, cSatzZaehler);                                                    
	eeAddress += sizeof(unsigned char); //Move address to the next byte after unsigned char 'c' (1 Byte)
	EEPROM.put(eeAddress, cWasserGrenze);                                                     
	eeAddress += sizeof(unsigned char); //Move address to the next byte after unsigned char 'c' (1 Byte)
	EEPROM.put(eeAddress, cSatzGrenze);                                                    
	eeAddress += sizeof(unsigned char); //Move address to the next byte after unsigned char 'c' (1 Byte)
	EEPROM.put(eeAddress, bTasseZeigen);
	eeAddress += sizeof(bool); 			//Move address to the next byte after bool 'b' (1 Byte?)
	EEPROM.put(eeAddress, iTasseFullZeit);
	eeAddress += sizeof(int); 			//Move address to the next byte after int 'i' (2 Byte?)
	EEPROM.put(eeAddress, iTasseWarteZeit);
	eeAddress += sizeof(int); 			//Move address to the next byte after int 'i' (2 Byte?)
	EEPROM.put(eeAddress, iTasseStartZeit);
	eeAddress += sizeof(int); 			//Move address to the next byte after int 'i' (2 Byte?)
}
// ----------------------------------------------------------------------------------------------------------------------------

// ----------------------------------------------------------------------------------------------------------------------------
// Variablen aus EEPROM lesen
// ----------------------------------------------------------------------------------------------------------------------------
void readFromEEPROM()
{	// alle Werte werden immer auf einmal behandelt, per EEPROM.get() 
	// Damit das funktioniert, muss save und read immer in der gleichen Reihenfolge passieren
	int eeAddress = EESTARTADRESSE;	// immmer auf Adresse 0 starten (fuer Debug: Adresse 1000)
	
	EEPROM.get(eeAddress, iGezapfteTassen);
	eeAddress += sizeof(int); 			//Move address to the next byte after int 'i' (2 Byte)
	EEPROM.get(eeAddress, cWasserZaehler);
	eeAddress += sizeof(unsigned char); //Move address to the next byte after unsigned char 'c' (1 Byte)
	EEPROM.get(eeAddress, cSatzZaehler);
	eeAddress += sizeof(unsigned char); //Move address to the next byte after unsigned char 'c' (1 Byte)
	EEPROM.get(eeAddress, cWasserGrenze);
	eeAddress += sizeof(unsigned char); //Move address to the next byte after unsigned char 'c' (1 Byte)
	EEPROM.get(eeAddress, cSatzGrenze);
	eeAddress += sizeof(unsigned char); //Move address to the next byte after unsigned char 'c' (1 Byte)
	EEPROM.get(eeAddress, bTasseZeigen);
	eeAddress += sizeof(bool); 			//Move address to the next byte after bool 'b' (1 Byte?)
	EEPROM.get(eeAddress, iTasseFullZeit);
	eeAddress += sizeof(int); 			//Move address to the next byte after int 'i' (2 Byte?)
	EEPROM.get(eeAddress, iTasseWarteZeit);
	eeAddress += sizeof(int); 			//Move address to the next byte after int 'i' (2 Byte?)
	EEPROM.get(eeAddress, iTasseStartZeit);
	eeAddress += sizeof(int); 			//Move address to the next byte after int 'i' (2 Byte?)
}
// EEPROM Inhalt ab 1000(dez)
// DB 14 	-> 14 DB = 5339 (erhoeht sich mit jeder gezapften Tasse)
// 01 		-> 1	(variiert)
// 09 		-> 9	(variiert)
// 0E 		-> 15 	(wird über Menue eingestellt)
// 17 		-> 31 	(wird über Menue eingestellt)
// 00000001
// ----------------------------------------------------------------------------------------------------------------------------

// ----------------------------------------------------------------------------------------------------------------------------
// Warnungen der beiden Behaelter anzeigen
// ----------------------------------------------------------------------------------------------------------------------------
void displayWarnungen()
{
	// Warnungen pulsieren lassen, 2020-11-26
	if (DimDirection == UP)										// Heller werden
	{
		if (cAktuelleHelligkeit < 140) cAktuelleHelligkeit += 10;// Anschlag..., max. 245 bei Step 10, 2020-11-27
		else DimDirection = DOWN;								// ...-> Richtung wechseln
	}
	else														// Dunkler werden
	{
		if (cAktuelleHelligkeit > 10) cAktuelleHelligkeit -= 10;	// Anschlag..., min. 10 bei Step 10, 2020-11-27
		else DimDirection = UP;									// ...-> Richtung wechseln
	}
	//setDisplayWert(cAktuelleHelligkeit);
	display.ssd1306_command(0x81);								// Kontrast Command
	display.ssd1306_command(cAktuelleHelligkeit);				// Helligkeit
	lWarnPulseZeit = millis();
	// \Warnungen pulsieren lassen, 2020-11-26

	// Logik fuer Warnungen
	// Wasserstand
	if(cWasserZaehler > (cWasserGrenze-1))				// -1 weil sonst eine Tasse mehr gezapft wird
	{
		display.fillRect(0,0,128,64, BLACK);			// komplettes Display loeschen // top-left X, top-left Y, Width, Height
		// display.fillRect(0,16,128,48, BLACK);		// unteres Display loeschen // top-left X, top-left Y, Width, Height
		display.setTextColor(WHITE);						// set text color
		display.setTextSize(2);								// set text size
		display.setCursor(TEXTLEFT+28,TEXTTOP);				// set text cursor position	
		display.print(F("Wasser"));							// show text
		display.setCursor(TEXTLEFT+4,TEXTTOP+26);			// set text cursor position	
		display.print(F("nachf"));
		display.print(char(129));							// ü
		display.print(F("llen"));							// show text
		bWarnung = true;									// unterdrueckt die erste Zeile
	}
	// Kaffeesatz
	else if(cSatzZaehler > (cSatzGrenze-1))				// -1 weil sonst eine Tasse mehr gezapft wird
	{
		display.fillRect(0,0,128,64, BLACK);			// komplettes Display loeschen // top-left X, top-left Y, Width, Height
		// display.fillRect(0,16,128,48, BLACK);		// unteres Display loeschen // top-left X, top-left Y, Width, Height
		display.setTextColor(WHITE);						// set text color
		display.setTextSize(2);								// set text size
		display.setCursor(TEXTLEFT+4,TEXTTOP);				// set text cursor position	
		display.print(F("Kaffeesatz"));						// show text
		display.setCursor(TEXTLEFT+28,TEXTTOP+26);			// set text cursor position	
		display.print(F("leeren"));							// show text
		bWarnung = true;									// unterdrueckt die erste Zeile
	}
}
// ----------------------------------------------------------------------------------------------------------------------------
// Fuellstaende der beiden Behaelter als Grafik anzeigen
// ----------------------------------------------------------------------------------------------------------------------------
void displayFuellStaende()
{	
	// sporadisch Smiley statt Fuellstaende anzeigen
	if((millis() > lSmileyPause) && (bSmileySichtbar == false))		// Smiley soll jetzt angezeigt werden
	{
		display.fillRect(0,16,128,48, BLACK);		// unteres Display loeschen // top-left X, top-left Y, Width, Height
		// digitalWrite(GELBELED, HIGH);			// Debug
		
		bCheckLastSmiley = true;
		// Smiley anzeigen		// void drawBitmap(TopLeft x, TopLeft y, *bitmap, width, height, color);
		displaySmiley();
		bSmileySichtbar = true;
		
		lSmileyAnzeige = random(SMILEYDAMIN, SMILEYDAMAX);			// Smiley bleibt 3..6 Sekunden sichtbar			
		
		lSmileyAnzeige += millis();
	}	

	if((millis() > lSmileyAnzeige) && (bSmileySichtbar == true))
	{
		// naechstes Auftauchen des Smiley bestimmen
		lSmileyPause = random(SMILEYWEGMIN, SMILEYWEGMAX);	// Smiley kommt alle 10..65 Sekunden
		bSmileySichtbar = false;
		
		cSmileySelect = random(1, 11);			// welcher Smiley wird als naechstes angezeigt [1..10], random geht bis max-1
		
		lSmileyPause += millis();				// Pause zaehlt ab jetzt
	}
		
	if(bSmileySichtbar == false)				// kein Smiley -> Fuellstaende als Balkengrafik anzeigen
	{
		// display.fillRect(0,0,128,64, BLACK);		// komplettes Display loeschen // top-left X, top-left Y, Width, Height
 		display.fillRect(0,16,128,48, BLACK);		// unteres Display loeschen // top-left X, top-left Y, Width, Height
		
		// ------------------------------------------------------------
		// Grafikausgaben im unteren Teil des Display
		// ------------------------------------------------------------
		// Rahmen fuer Wasserstand
		display.drawRect(TOPLEFTWASSER,16,40,48, WHITE); 	// top-left X, top-left Y, Width, Height
		
		// die Berechnungen fuer die Fuellstaende muessen mit (float) durchgefuehrt werden, 
		// sonst stimmt die Darstellung nicht mit den Zaehlern ueberein
		
		// Fuellung fuer Wasserstand
		display.fillRect(TOPLEFTWASSER+1,17,38,46, BLACK);
		fuellWasser = (((float)cWasserZaehler * (float)((float)((float)cAnzeigeMax - (float)cAnzeigeMin) / (float)cWasserGrenze)) + cAnzeigeMin);
		display.fillRect(TOPLEFTWASSER+1,fuellWasser,38,46 - (fuellWasser - cAnzeigeMin), WHITE);
		
		// Rahmen fuer Kaffeesatz
		display.drawRect(TOPLEFTSATZ,16,40,48, WHITE); 	// top-left X, top-left Y, Width, Height
		
		// Fuellung fuer Kaffeesatz
		display.fillRect(TOPLEFTSATZ+1,17,38,46, BLACK);
		fuellSatz = (cAnzeigeMax - (cSatzZaehler * (float)((float)((float)cAnzeigeMax - (float)cAnzeigeMin) / (float)cSatzGrenze))) ;
		display.fillRect(TOPLEFTSATZ+1,fuellSatz,38,46 - (fuellSatz - cAnzeigeMin), WHITE);
		bWarnung = false;						// unterdrueckt die erste Zeile nicht
		
		// digitalWrite(GELBELED, LOW);			// Debug
	}
}

// ----------------------------------------------------------------------------------------------------------------------------
// sporadisch Smileys anzeigen
// ----------------------------------------------------------------------------------------------------------------------------
void displaySmiley()
{
	// verhindern, dass zweimal hintereinander derselbe Smiley angezeigt wird
	if((bCheckLastSmiley == true) && (cSmileyLast == cSmileySelect))
	{
		if(cSmileySelect < 10) cSmileySelect++; else if(cSmileySelect > 1) cSmileySelect--;
	}

	switch(cSmileySelect)										// Smiley aussuchen
	{
		case 1: 
			pSmileyAdresse = Smiley1_bw;
			break;
		case 2:
			pSmileyAdresse = Smiley2_bw;
			break;
		case 3:
			pSmileyAdresse = Smiley5_bw;
			break;
		case 4:
			pSmileyAdresse = Smiley6_bw;
			break;
		case 5:
			pSmileyAdresse = Smiley7_bw;
			break;
		case 6:
			pSmileyAdresse = Smiley8_bw;
			break;
		case 7:
			pSmileyAdresse = Smiley9_bw;
			break;
		case 8:
			pSmileyAdresse = Smiley10_bw;
			break;
		case 9:
			pSmileyAdresse = Smiley11_bw;
			break;
		case 10:
			pSmileyAdresse = Smiley12_bw;
			break;
		case 11:												// Version ausgeben, 2020-11-27
			pSmileyAdresse = Smiley13_bw;
			break;
		default:	
			pSmileyAdresse = Smiley1_bw;
			break;
	}
	
	cSmileyLast = cSmileySelect;								// angezeigten Smiley merken
	
	if (pSmileyAdresse == Smiley13_bw)							// Version ausgeben, 2020-11-27
	{
		display.setCursor(TEXTLEFT+14, ZEILE3);					// set text cursor position
		display.print("QuickMill Saver");						// show text
		display.setCursor(TEXTLEFT, ZEILE4);					// set text cursor position
		display.print("Version: ");								// show text
		display.print(VERSION);									// show version
		display.setCursor(TEXTLEFT, ZEILE5);					// set text cursor position
		display.print("Build:   ");								// show text
		display.print(BUILDDATE);								// show Datum
	}
	else display.drawBitmap(TOPLEFTSMILEY, 16, pSmileyAdresse, 48, 48, WHITE);	// Smiley ausgeben

}

// ----------------------------------------------------------------------------------------------------------------------------
// Werte der QTouch Tasten anzeigen
// ----------------------------------------------------------------------------------------------------------------------------
void showQTvalues()
{
	lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
												// der Timer muss bei jedem Tastendruck nachgetriggert werden
	display.setCursor(TEXTLEFT,ZEILE2);			// set text cursor position	("-2" == 2. Zeile)
	display.print(F("Key 0 Hoch"));				// show text
	display.setCursor(TEXTLEFT+84,ZEILE2);		// set text cursor position
	display.print(QTouch.getSignalForKey(0));	// Wert ausgeben
	
	display.setCursor(TEXTLEFT,ZEILE3);			// set text cursor position	("-2" == 2. Zeile)
	display.print(F("Key 1 Runter"));			// show text
	display.setCursor(TEXTLEFT+84,ZEILE3);		// set text cursor position
	display.print(QTouch.getSignalForKey(1));	// Wert ausgeben
	
	display.setCursor(TEXTLEFT,ZEILE4);			// set text cursor position	("-2" == 2. Zeile)
	display.print(F("Key 2 Links"));			// show text
	display.setCursor(TEXTLEFT+84,ZEILE4);		// set text cursor position
	display.print(QTouch.getSignalForKey(2));	// Wert ausgeben
	
	display.setCursor(TEXTLEFT,ZEILE5);			// set text cursor position	("-2" == 2. Zeile)
	display.print(F("Key 3 Rechts"));			// show text
	display.setCursor(TEXTLEFT+84,ZEILE5);		// set text cursor position
	display.print(QTouch.getSignalForKey(3));	// Wert ausgeben
	
}

// ----------------------------------------------------------------------------------------------------------------------------
// Rahmen zum Ausrichten des Display anzeigen
// ----------------------------------------------------------------------------------------------------------------------------
void showFrames()
{
	lMenuTimeout = millis();					// Timer um das Menue nach 15 s Inaktivitaet wieder zu verlassen
												// der Timer muss bei jedem Tastendruck nachgetriggert werden
												// hier _verhindert_ der Aufruf das Verlassen nach 15 Sekunden
	if(cFrameSelect == 1)						
	{
		display.drawRect(0, 0, 128, 64, WHITE);	// Rahmen darstellen
	}
	else
	{
		display.fillRect(0, 0, 128, 64, WHITE);	// Flaeche darstellen
	}
}

// ----------------------------------------------------------------------------------------------------------------------------
// Display dimmen um Einbrennen zu minimieren
// ----------------------------------------------------------------------------------------------------------------------------
void setDisplayDim()
{
	display.ssd1306_command(0x81);					// Kontrast Command
	display.ssd1306_command(1);						// relativ dunkel
}

// ----------------------------------------------------------------------------------------------------------------------------
// Display auf normale Helligkeit einstellen
// ----------------------------------------------------------------------------------------------------------------------------
void setDisplayNorm()
{
	display.ssd1306_command(0x81);					// Kontrast Command
	display.ssd1306_command(127);					// default Helligkeit
	lDimDisplayZeit = millis();						// und 2 Minuten Timer neu starten
}

// ----------------------------------------------------------------------------------------------------------------------------
// Display pulsieren
// ----------------------------------------------------------------------------------------------------------------------------
void setDisplayWert(uint8_t value)
{
	display.ssd1306_command(0x81);					// Kontrast Command
	display.ssd1306_command(value);					// Helligkeit
}


// ----------------------------------------------------------------------------------------------------------------------------
// function to configure the watchdog: let it sleep 8 seconds before firing
// when firing, configure it for resuming program execution by default
// hangs will be determined in the ISR function
// ----------------------------------------------------------------------------------------------------------------------------
// Watchdog konfigurieren
// ----------------------------------------------------------------------------------------------------------------------------
/* void configure_wdt(void)
{
  cli();                           		// disable interrupts for changing the registers

  MCUSR = 0;                       		// reset status register flags
  
  // Put timer in interrupt-only mode:                                       
  WDTCSR |= 0b00011000;            		// Set WDCE (5th from left) and WDE (4th from left) to enter config mode,
										// using bitwise OR assignment (leaves other bits unchanged).
  WDTCSR =  0b01000000 | 0b00100001; 	// set WDIE: interrupt enabled
										// clr WDE: reset disabled
										// and set delay interval (right side of bar) to 8 seconds

  sei();                           		// re-enable interrupts 
}
 */
// interrupt raised by the watchdog firing
// when the watchdog fires, this function will be executed
// remember that interrupts are disabled in ISR functions
// now we must check if the board is sleeping or if this is a genuine
// interrupt (hanging)
// ----------------------------------------------------------------------------------------------------------------------------
// Watchdog ISR
// ----------------------------------------------------------------------------------------------------------------------------
/* ISR(WDT_vect)
{
    {
        // must be rebooted
        // configure
        MCUSR = 0;                          // reset flags
       
        // Put timer in reset-only mode:
        WDTCSR |= 0b00011000;               // Enter config mode.
        WDTCSR =  0b00001000 | 0b00000111;  // clr WDIE (interrupt enable...7th from left)
                                            // set WDE (reset enable...4th from left), and set delay interval
                                            // reset system in 2 s...
                                            // unless wdt_disable() in loop() is reached first
        // reboot
        // while(1);
    }
}
// ----------------------------------------------------------------------------------------------------------------------------
 */